import { ExtendedCluster, PrivateKey } from './util';
import { ChannelData } from './solana/models/ChannelData';
import { CEKAccountData } from './solana/models/CEKAccountData';
import { PublicKey } from '@solana/web3.js';
import { VerificationMethod } from 'did-resolver';
import { CEKData } from './solana/models/CEKData';
export declare type MessageSender = {
    did: string;
    alias?: string;
};
export declare class Message {
    readonly sender: MessageSender;
    readonly content: string;
    readonly timestamp: number;
    constructor(sender: MessageSender, content: string, timestamp: number);
    displayableSender(): string;
    withContent(content: string): Message;
    static build(senderDIDKey: PublicKey, content: string, timestamp: number, cluster?: ExtendedCluster): Promise<Message>;
}
export declare class Channel {
    readonly name: string;
    readonly messages: Message[];
    readonly address: PublicKey;
    private cek?;
    private cluster?;
    constructor(name: string, messages: Message[], address: PublicKey, cek?: Uint8Array | undefined, cluster?: "devnet" | "testnet" | "mainnet-beta" | "localnet" | undefined);
    encrypt(message: string): Promise<string>;
    encryptCEKForDID(did: string): Promise<CEKData[]>;
    encryptCEK(verificationMethod: VerificationMethod): Promise<CEKData>;
    hasMember(did: PublicKey): Promise<boolean>;
    static fromChainData(address: PublicKey, channelData: ChannelData, cekAccountData: CEKAccountData, memberDID: string, memberKey?: PrivateKey, cluster?: ExtendedCluster): Promise<Channel>;
}
