import { Enum, Assignable } from './solanaBorsh';
import { PublicKey, TransactionInstruction } from '@solana/web3.js';
import { CEKData } from './models/CEKData';
import { MessageData } from './models/MessageData';
export declare class InitializeChannel extends Assignable {
    name: string;
    CEKs: CEKData[];
}
export declare class InitializeDirectChannel extends Assignable {
    creatorCEKs: CEKData[];
    inviteeCEKs: CEKData[];
}
export declare class Post extends Assignable {
    message: string;
}
export declare class AddToChannel extends Assignable {
    CEKs: CEKData[];
}
export declare class AddCEK extends Assignable {
    cek: CEKData;
}
export declare class RemoveCEK extends Assignable {
    kid: string;
}
export declare class CreateUserDetails extends Assignable {
    alias: string;
    addressBook: string;
    size: number;
}
export declare class UpdateUserDetails extends Assignable {
    alias: string;
    addressBook: string;
}
export declare class SolariumInstruction extends Enum {
    initializeChannel: InitializeChannel;
    initializeDirectChannel: InitializeDirectChannel;
    post: Post;
    addToChannel: AddToChannel;
    addCEK: AddCEK;
    removeCEK: RemoveCEK;
    createUserDetails: CreateUserDetails;
    updateUserDetails: UpdateUserDetails;
    static initializeChannel(name: string, CEKs: CEKData[]): SolariumInstruction;
    static initializeDirectChannel(creatorCEKs: CEKData[], inviteeCEKs: CEKData[]): SolariumInstruction;
    static post(message: string): SolariumInstruction;
    static addToChannel(CEKs: CEKData[]): SolariumInstruction;
    static addCEK(cek: CEKData): SolariumInstruction;
    static removeCEK(kid: string): SolariumInstruction;
    static createUserDetails(alias: string, addressBook?: string, size?: number): SolariumInstruction;
    static updateUserDetails(alias: string, addressBook?: string): SolariumInstruction;
}
export declare function getCekAccountKey(ownerDID: PublicKey, channel: PublicKey): Promise<PublicKey>;
export declare function getUserDetailsKey(did: PublicKey): Promise<PublicKey>;
export declare function getDirectChannelAccountKey(did1: PublicKey, did2: PublicKey): Promise<PublicKey>;
export declare function initializeChannel(payer: PublicKey, channel: PublicKey, name: string, creatorDID: PublicKey, creatorAuthority: PublicKey, CEKs: CEKData[]): Promise<TransactionInstruction>;
export declare function initializeDirectChannel(payer: PublicKey, channel: PublicKey, creatorDID: PublicKey, creatorAuthority: PublicKey, inviteeDID: PublicKey, creatorCEKs: CEKData[], inviteeCEKs: CEKData[]): Promise<TransactionInstruction>;
export declare function post(channel: PublicKey, senderAuthority: PublicKey, message: MessageData): Promise<TransactionInstruction>;
export declare function addToChannel(payer: PublicKey, channel: PublicKey, inviteeDID: PublicKey, inviterDID: PublicKey, inviterAuthority: PublicKey, CEKs: CEKData[]): Promise<TransactionInstruction>;
export declare function addCEK(channel: PublicKey, ownerDID: PublicKey, ownerAuthority: PublicKey, cek: CEKData): Promise<TransactionInstruction>;
export declare function removeCEK(channel: PublicKey, ownerDID: PublicKey, ownerAuthority: PublicKey, kid: string): Promise<TransactionInstruction>;
export declare function createUserDetails(payer: PublicKey, did: PublicKey, authority: PublicKey, alias: string, size?: number): Promise<TransactionInstruction>;
export declare function updateUserDetails(did: PublicKey, authority: PublicKey, alias: string, addressBook: string): Promise<TransactionInstruction>;
