import { Connection, Keypair, PublicKey, TransactionInstruction } from '@solana/web3.js';
import { SignCallback } from '../wallet';
import { ExtendedCluster } from '../util';
import { CEKData } from './models/CEKData';
import { ChannelData } from './models/ChannelData';
import { CEKAccountData } from './models/CEKAccountData';
import { UserDetailsData } from './models/UserDetailsData';
export declare class SolariumTransaction {
    static createGroupChannel(connection: Connection, payer: PublicKey, creatorDID: PublicKey, creatorAuthority: PublicKey, name: string, initialCEKs: CEKData[], signCallback: SignCallback, cluster?: ExtendedCluster): Promise<PublicKey>;
    static createDirectChannel(payer: PublicKey, creatorDID: PublicKey, creatorAuthority: PublicKey, inviteeDID: PublicKey, creatorCEKs: CEKData[], inviteeCEKs: CEKData[], signCallback: SignCallback, cluster?: ExtendedCluster): Promise<PublicKey>;
    static addDIDToChannel(payer: PublicKey, channel: PublicKey, inviterDID: PublicKey, inviterAuthority: PublicKey, inviteeDID: PublicKey, ceks: CEKData[], signCallback: SignCallback, cluster?: ExtendedCluster): Promise<void>;
    static getChannelData(connection: Connection, channel: PublicKey): Promise<ChannelData | null>;
    static getCEKAccountData(connection: Connection, ownerDID: PublicKey, channel: PublicKey): Promise<CEKAccountData | null>;
    static postMessage(channel: PublicKey, senderDID: PublicKey, senderAuthority: PublicKey, message: string, signCallback: SignCallback, cluster?: ExtendedCluster): Promise<string>;
    static addCEKToAccount(channel: PublicKey, memberDID: PublicKey, memberAuthority: PublicKey, cek: CEKData, signCallback: SignCallback, cluster?: ExtendedCluster): Promise<void>;
    static createUserDetails(payer: PublicKey, did: PublicKey, authority: PublicKey, signCallback: SignCallback, alias: string, size?: number, cluster?: ExtendedCluster): Promise<PublicKey>;
    static updateUserDetails(did: PublicKey, authority: PublicKey, signCallback: SignCallback, alias: string, addressBook: string, cluster?: ExtendedCluster): Promise<PublicKey>;
    static getUserDetails(connection: Connection, did: PublicKey): Promise<UserDetailsData | null>;
    static signAndSendTransaction(instructions: TransactionInstruction[], createSignedTx: SignCallback, additionalSigners: Keypair[], cluster?: ExtendedCluster): Promise<string>;
}
