import { Connection, Keypair, PublicKey } from '@solana/web3.js';
import { SignCallback } from '../lib/wallet';
import { AddressBook, UserDetails } from '../lib/UserDetails';
import { SolariumCache } from '../lib/cache';
export declare const userDetailsCache: SolariumCache<UserDetails | null, (key: string, connection: Connection) => Promise<UserDetails | null>>;
export declare const getUserDetails: (key: string, skipCache: boolean, connection: Connection) => Promise<UserDetails | null>;
/**
 * Create a Solarium user details account for a DID
 * @param did
 * @param owner
 * @param payer
 * @param alias
 * @param size
 * @param signCallback
 * @param cluster
 */
export declare const createUserDetails: (did: string, owner: Keypair | PublicKey, payer: Keypair | PublicKey, alias: string, size?: number | undefined, signCallback?: SignCallback | undefined, cluster?: "devnet" | "testnet" | "mainnet-beta" | "localnet" | undefined) => Promise<void>;
/**
 * Update a Solarium user details account
 * @param did
 * @param connection
 * @param owner
 * @param payer
 * @param alias
 * @param addressBook
 * @param signCallback
 * @param cluster
 */
export declare const updateUserDetails: (did: string, connection: Connection, owner: Keypair | PublicKey, payer: Keypair | PublicKey, alias?: string | undefined, addressBook?: AddressBook | undefined, signCallback?: SignCallback | undefined, cluster?: "devnet" | "testnet" | "mainnet-beta" | "localnet" | undefined) => Promise<void>;
