import { PublicKey, Keypair, clusterApiUrl, Connection, sendAndConfirmRawTransaction, TransactionInstruction, SYSVAR_RENT_PUBKEY, SystemProgram, Transaction } from '@solana/web3.js';
import { decode } from 'bs58';
import { ClusterType, DecentralizedIdentifier, keyToIdentifier, resolve, createRegisterInstruction, createUpdateInstruction } from '@identity.com/sol-did-client';
export { keyToIdentifier } from '@identity.com/sol-did-client';
import Debug from 'debug';
import { memoizeWith, uniqWith } from 'ramda';
import { serialize, BorshError, BinaryReader } from 'borsh';
import { hash } from '@stablelib/sha256';
import { toString, fromString, concat } from 'uint8arrays';
import nacl from 'tweetnacl';
import { randomBytes } from '@stablelib/random';
import { NONCE_LENGTH, TAG_LENGTH, XChaCha20Poly1305 } from '@stablelib/xchacha20poly1305';
import { generateKeyPair, sharedKey } from '@stablelib/x25519';
import { convertPublicKey, convertSecretKey } from 'ed2curve-esm';
import { Observable, from, merge } from 'rxjs';
import { switchMap, distinct } from 'rxjs/operators';

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;

  _setPrototypeOf(subClass, superClass);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];
  if (it) return (it = it.call(o)).next.bind(it);

  if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
    if (it) o = it;
    var i = 0;
    return function () {
      if (i >= o.length) return {
        done: true
      };
      return {
        done: false,
        value: o[i++]
      };
    };
  }

  throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var runtime_1 = createCommonjsModule(function (module) {
/**
 * Copyright (c) 2014-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

var runtime = (function (exports) {

  var Op = Object.prototype;
  var hasOwn = Op.hasOwnProperty;
  var undefined$1; // More compressible than void 0.
  var $Symbol = typeof Symbol === "function" ? Symbol : {};
  var iteratorSymbol = $Symbol.iterator || "@@iterator";
  var asyncIteratorSymbol = $Symbol.asyncIterator || "@@asyncIterator";
  var toStringTagSymbol = $Symbol.toStringTag || "@@toStringTag";

  function define(obj, key, value) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
    return obj[key];
  }
  try {
    // IE 8 has a broken Object.defineProperty that only works on DOM objects.
    define({}, "");
  } catch (err) {
    define = function(obj, key, value) {
      return obj[key] = value;
    };
  }

  function wrap(innerFn, outerFn, self, tryLocsList) {
    // If outerFn provided and outerFn.prototype is a Generator, then outerFn.prototype instanceof Generator.
    var protoGenerator = outerFn && outerFn.prototype instanceof Generator ? outerFn : Generator;
    var generator = Object.create(protoGenerator.prototype);
    var context = new Context(tryLocsList || []);

    // The ._invoke method unifies the implementations of the .next,
    // .throw, and .return methods.
    generator._invoke = makeInvokeMethod(innerFn, self, context);

    return generator;
  }
  exports.wrap = wrap;

  // Try/catch helper to minimize deoptimizations. Returns a completion
  // record like context.tryEntries[i].completion. This interface could
  // have been (and was previously) designed to take a closure to be
  // invoked without arguments, but in all the cases we care about we
  // already have an existing method we want to call, so there's no need
  // to create a new function object. We can even get away with assuming
  // the method takes exactly one argument, since that happens to be true
  // in every case, so we don't have to touch the arguments object. The
  // only additional allocation required is the completion record, which
  // has a stable shape and so hopefully should be cheap to allocate.
  function tryCatch(fn, obj, arg) {
    try {
      return { type: "normal", arg: fn.call(obj, arg) };
    } catch (err) {
      return { type: "throw", arg: err };
    }
  }

  var GenStateSuspendedStart = "suspendedStart";
  var GenStateSuspendedYield = "suspendedYield";
  var GenStateExecuting = "executing";
  var GenStateCompleted = "completed";

  // Returning this object from the innerFn has the same effect as
  // breaking out of the dispatch switch statement.
  var ContinueSentinel = {};

  // Dummy constructor functions that we use as the .constructor and
  // .constructor.prototype properties for functions that return Generator
  // objects. For full spec compliance, you may wish to configure your
  // minifier not to mangle the names of these two functions.
  function Generator() {}
  function GeneratorFunction() {}
  function GeneratorFunctionPrototype() {}

  // This is a polyfill for %IteratorPrototype% for environments that
  // don't natively support it.
  var IteratorPrototype = {};
  IteratorPrototype[iteratorSymbol] = function () {
    return this;
  };

  var getProto = Object.getPrototypeOf;
  var NativeIteratorPrototype = getProto && getProto(getProto(values([])));
  if (NativeIteratorPrototype &&
      NativeIteratorPrototype !== Op &&
      hasOwn.call(NativeIteratorPrototype, iteratorSymbol)) {
    // This environment has a native %IteratorPrototype%; use it instead
    // of the polyfill.
    IteratorPrototype = NativeIteratorPrototype;
  }

  var Gp = GeneratorFunctionPrototype.prototype =
    Generator.prototype = Object.create(IteratorPrototype);
  GeneratorFunction.prototype = Gp.constructor = GeneratorFunctionPrototype;
  GeneratorFunctionPrototype.constructor = GeneratorFunction;
  GeneratorFunction.displayName = define(
    GeneratorFunctionPrototype,
    toStringTagSymbol,
    "GeneratorFunction"
  );

  // Helper for defining the .next, .throw, and .return methods of the
  // Iterator interface in terms of a single ._invoke method.
  function defineIteratorMethods(prototype) {
    ["next", "throw", "return"].forEach(function(method) {
      define(prototype, method, function(arg) {
        return this._invoke(method, arg);
      });
    });
  }

  exports.isGeneratorFunction = function(genFun) {
    var ctor = typeof genFun === "function" && genFun.constructor;
    return ctor
      ? ctor === GeneratorFunction ||
        // For the native GeneratorFunction constructor, the best we can
        // do is to check its .name property.
        (ctor.displayName || ctor.name) === "GeneratorFunction"
      : false;
  };

  exports.mark = function(genFun) {
    if (Object.setPrototypeOf) {
      Object.setPrototypeOf(genFun, GeneratorFunctionPrototype);
    } else {
      genFun.__proto__ = GeneratorFunctionPrototype;
      define(genFun, toStringTagSymbol, "GeneratorFunction");
    }
    genFun.prototype = Object.create(Gp);
    return genFun;
  };

  // Within the body of any async function, `await x` is transformed to
  // `yield regeneratorRuntime.awrap(x)`, so that the runtime can test
  // `hasOwn.call(value, "__await")` to determine if the yielded value is
  // meant to be awaited.
  exports.awrap = function(arg) {
    return { __await: arg };
  };

  function AsyncIterator(generator, PromiseImpl) {
    function invoke(method, arg, resolve, reject) {
      var record = tryCatch(generator[method], generator, arg);
      if (record.type === "throw") {
        reject(record.arg);
      } else {
        var result = record.arg;
        var value = result.value;
        if (value &&
            typeof value === "object" &&
            hasOwn.call(value, "__await")) {
          return PromiseImpl.resolve(value.__await).then(function(value) {
            invoke("next", value, resolve, reject);
          }, function(err) {
            invoke("throw", err, resolve, reject);
          });
        }

        return PromiseImpl.resolve(value).then(function(unwrapped) {
          // When a yielded Promise is resolved, its final value becomes
          // the .value of the Promise<{value,done}> result for the
          // current iteration.
          result.value = unwrapped;
          resolve(result);
        }, function(error) {
          // If a rejected Promise was yielded, throw the rejection back
          // into the async generator function so it can be handled there.
          return invoke("throw", error, resolve, reject);
        });
      }
    }

    var previousPromise;

    function enqueue(method, arg) {
      function callInvokeWithMethodAndArg() {
        return new PromiseImpl(function(resolve, reject) {
          invoke(method, arg, resolve, reject);
        });
      }

      return previousPromise =
        // If enqueue has been called before, then we want to wait until
        // all previous Promises have been resolved before calling invoke,
        // so that results are always delivered in the correct order. If
        // enqueue has not been called before, then it is important to
        // call invoke immediately, without waiting on a callback to fire,
        // so that the async generator function has the opportunity to do
        // any necessary setup in a predictable way. This predictability
        // is why the Promise constructor synchronously invokes its
        // executor callback, and why async functions synchronously
        // execute code before the first await. Since we implement simple
        // async functions in terms of async generators, it is especially
        // important to get this right, even though it requires care.
        previousPromise ? previousPromise.then(
          callInvokeWithMethodAndArg,
          // Avoid propagating failures to Promises returned by later
          // invocations of the iterator.
          callInvokeWithMethodAndArg
        ) : callInvokeWithMethodAndArg();
    }

    // Define the unified helper method that is used to implement .next,
    // .throw, and .return (see defineIteratorMethods).
    this._invoke = enqueue;
  }

  defineIteratorMethods(AsyncIterator.prototype);
  AsyncIterator.prototype[asyncIteratorSymbol] = function () {
    return this;
  };
  exports.AsyncIterator = AsyncIterator;

  // Note that simple async functions are implemented on top of
  // AsyncIterator objects; they just return a Promise for the value of
  // the final result produced by the iterator.
  exports.async = function(innerFn, outerFn, self, tryLocsList, PromiseImpl) {
    if (PromiseImpl === void 0) PromiseImpl = Promise;

    var iter = new AsyncIterator(
      wrap(innerFn, outerFn, self, tryLocsList),
      PromiseImpl
    );

    return exports.isGeneratorFunction(outerFn)
      ? iter // If outerFn is a generator, return the full iterator.
      : iter.next().then(function(result) {
          return result.done ? result.value : iter.next();
        });
  };

  function makeInvokeMethod(innerFn, self, context) {
    var state = GenStateSuspendedStart;

    return function invoke(method, arg) {
      if (state === GenStateExecuting) {
        throw new Error("Generator is already running");
      }

      if (state === GenStateCompleted) {
        if (method === "throw") {
          throw arg;
        }

        // Be forgiving, per 25.3.3.3.3 of the spec:
        // https://people.mozilla.org/~jorendorff/es6-draft.html#sec-generatorresume
        return doneResult();
      }

      context.method = method;
      context.arg = arg;

      while (true) {
        var delegate = context.delegate;
        if (delegate) {
          var delegateResult = maybeInvokeDelegate(delegate, context);
          if (delegateResult) {
            if (delegateResult === ContinueSentinel) continue;
            return delegateResult;
          }
        }

        if (context.method === "next") {
          // Setting context._sent for legacy support of Babel's
          // function.sent implementation.
          context.sent = context._sent = context.arg;

        } else if (context.method === "throw") {
          if (state === GenStateSuspendedStart) {
            state = GenStateCompleted;
            throw context.arg;
          }

          context.dispatchException(context.arg);

        } else if (context.method === "return") {
          context.abrupt("return", context.arg);
        }

        state = GenStateExecuting;

        var record = tryCatch(innerFn, self, context);
        if (record.type === "normal") {
          // If an exception is thrown from innerFn, we leave state ===
          // GenStateExecuting and loop back for another invocation.
          state = context.done
            ? GenStateCompleted
            : GenStateSuspendedYield;

          if (record.arg === ContinueSentinel) {
            continue;
          }

          return {
            value: record.arg,
            done: context.done
          };

        } else if (record.type === "throw") {
          state = GenStateCompleted;
          // Dispatch the exception by looping back around to the
          // context.dispatchException(context.arg) call above.
          context.method = "throw";
          context.arg = record.arg;
        }
      }
    };
  }

  // Call delegate.iterator[context.method](context.arg) and handle the
  // result, either by returning a { value, done } result from the
  // delegate iterator, or by modifying context.method and context.arg,
  // setting context.delegate to null, and returning the ContinueSentinel.
  function maybeInvokeDelegate(delegate, context) {
    var method = delegate.iterator[context.method];
    if (method === undefined$1) {
      // A .throw or .return when the delegate iterator has no .throw
      // method always terminates the yield* loop.
      context.delegate = null;

      if (context.method === "throw") {
        // Note: ["return"] must be used for ES3 parsing compatibility.
        if (delegate.iterator["return"]) {
          // If the delegate iterator has a return method, give it a
          // chance to clean up.
          context.method = "return";
          context.arg = undefined$1;
          maybeInvokeDelegate(delegate, context);

          if (context.method === "throw") {
            // If maybeInvokeDelegate(context) changed context.method from
            // "return" to "throw", let that override the TypeError below.
            return ContinueSentinel;
          }
        }

        context.method = "throw";
        context.arg = new TypeError(
          "The iterator does not provide a 'throw' method");
      }

      return ContinueSentinel;
    }

    var record = tryCatch(method, delegate.iterator, context.arg);

    if (record.type === "throw") {
      context.method = "throw";
      context.arg = record.arg;
      context.delegate = null;
      return ContinueSentinel;
    }

    var info = record.arg;

    if (! info) {
      context.method = "throw";
      context.arg = new TypeError("iterator result is not an object");
      context.delegate = null;
      return ContinueSentinel;
    }

    if (info.done) {
      // Assign the result of the finished delegate to the temporary
      // variable specified by delegate.resultName (see delegateYield).
      context[delegate.resultName] = info.value;

      // Resume execution at the desired location (see delegateYield).
      context.next = delegate.nextLoc;

      // If context.method was "throw" but the delegate handled the
      // exception, let the outer generator proceed normally. If
      // context.method was "next", forget context.arg since it has been
      // "consumed" by the delegate iterator. If context.method was
      // "return", allow the original .return call to continue in the
      // outer generator.
      if (context.method !== "return") {
        context.method = "next";
        context.arg = undefined$1;
      }

    } else {
      // Re-yield the result returned by the delegate method.
      return info;
    }

    // The delegate iterator is finished, so forget it and continue with
    // the outer generator.
    context.delegate = null;
    return ContinueSentinel;
  }

  // Define Generator.prototype.{next,throw,return} in terms of the
  // unified ._invoke helper method.
  defineIteratorMethods(Gp);

  define(Gp, toStringTagSymbol, "Generator");

  // A Generator should always return itself as the iterator object when the
  // @@iterator function is called on it. Some browsers' implementations of the
  // iterator prototype chain incorrectly implement this, causing the Generator
  // object to not be returned from this call. This ensures that doesn't happen.
  // See https://github.com/facebook/regenerator/issues/274 for more details.
  Gp[iteratorSymbol] = function() {
    return this;
  };

  Gp.toString = function() {
    return "[object Generator]";
  };

  function pushTryEntry(locs) {
    var entry = { tryLoc: locs[0] };

    if (1 in locs) {
      entry.catchLoc = locs[1];
    }

    if (2 in locs) {
      entry.finallyLoc = locs[2];
      entry.afterLoc = locs[3];
    }

    this.tryEntries.push(entry);
  }

  function resetTryEntry(entry) {
    var record = entry.completion || {};
    record.type = "normal";
    delete record.arg;
    entry.completion = record;
  }

  function Context(tryLocsList) {
    // The root entry object (effectively a try statement without a catch
    // or a finally block) gives us a place to store values thrown from
    // locations where there is no enclosing try statement.
    this.tryEntries = [{ tryLoc: "root" }];
    tryLocsList.forEach(pushTryEntry, this);
    this.reset(true);
  }

  exports.keys = function(object) {
    var keys = [];
    for (var key in object) {
      keys.push(key);
    }
    keys.reverse();

    // Rather than returning an object with a next method, we keep
    // things simple and return the next function itself.
    return function next() {
      while (keys.length) {
        var key = keys.pop();
        if (key in object) {
          next.value = key;
          next.done = false;
          return next;
        }
      }

      // To avoid creating an additional object, we just hang the .value
      // and .done properties off the next function object itself. This
      // also ensures that the minifier will not anonymize the function.
      next.done = true;
      return next;
    };
  };

  function values(iterable) {
    if (iterable) {
      var iteratorMethod = iterable[iteratorSymbol];
      if (iteratorMethod) {
        return iteratorMethod.call(iterable);
      }

      if (typeof iterable.next === "function") {
        return iterable;
      }

      if (!isNaN(iterable.length)) {
        var i = -1, next = function next() {
          while (++i < iterable.length) {
            if (hasOwn.call(iterable, i)) {
              next.value = iterable[i];
              next.done = false;
              return next;
            }
          }

          next.value = undefined$1;
          next.done = true;

          return next;
        };

        return next.next = next;
      }
    }

    // Return an iterator with no values.
    return { next: doneResult };
  }
  exports.values = values;

  function doneResult() {
    return { value: undefined$1, done: true };
  }

  Context.prototype = {
    constructor: Context,

    reset: function(skipTempReset) {
      this.prev = 0;
      this.next = 0;
      // Resetting context._sent for legacy support of Babel's
      // function.sent implementation.
      this.sent = this._sent = undefined$1;
      this.done = false;
      this.delegate = null;

      this.method = "next";
      this.arg = undefined$1;

      this.tryEntries.forEach(resetTryEntry);

      if (!skipTempReset) {
        for (var name in this) {
          // Not sure about the optimal order of these conditions:
          if (name.charAt(0) === "t" &&
              hasOwn.call(this, name) &&
              !isNaN(+name.slice(1))) {
            this[name] = undefined$1;
          }
        }
      }
    },

    stop: function() {
      this.done = true;

      var rootEntry = this.tryEntries[0];
      var rootRecord = rootEntry.completion;
      if (rootRecord.type === "throw") {
        throw rootRecord.arg;
      }

      return this.rval;
    },

    dispatchException: function(exception) {
      if (this.done) {
        throw exception;
      }

      var context = this;
      function handle(loc, caught) {
        record.type = "throw";
        record.arg = exception;
        context.next = loc;

        if (caught) {
          // If the dispatched exception was caught by a catch block,
          // then let that catch block handle the exception normally.
          context.method = "next";
          context.arg = undefined$1;
        }

        return !! caught;
      }

      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        var record = entry.completion;

        if (entry.tryLoc === "root") {
          // Exception thrown outside of any try block that could handle
          // it, so set the completion value of the entire function to
          // throw the exception.
          return handle("end");
        }

        if (entry.tryLoc <= this.prev) {
          var hasCatch = hasOwn.call(entry, "catchLoc");
          var hasFinally = hasOwn.call(entry, "finallyLoc");

          if (hasCatch && hasFinally) {
            if (this.prev < entry.catchLoc) {
              return handle(entry.catchLoc, true);
            } else if (this.prev < entry.finallyLoc) {
              return handle(entry.finallyLoc);
            }

          } else if (hasCatch) {
            if (this.prev < entry.catchLoc) {
              return handle(entry.catchLoc, true);
            }

          } else if (hasFinally) {
            if (this.prev < entry.finallyLoc) {
              return handle(entry.finallyLoc);
            }

          } else {
            throw new Error("try statement without catch or finally");
          }
        }
      }
    },

    abrupt: function(type, arg) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.tryLoc <= this.prev &&
            hasOwn.call(entry, "finallyLoc") &&
            this.prev < entry.finallyLoc) {
          var finallyEntry = entry;
          break;
        }
      }

      if (finallyEntry &&
          (type === "break" ||
           type === "continue") &&
          finallyEntry.tryLoc <= arg &&
          arg <= finallyEntry.finallyLoc) {
        // Ignore the finally entry if control is not jumping to a
        // location outside the try/catch block.
        finallyEntry = null;
      }

      var record = finallyEntry ? finallyEntry.completion : {};
      record.type = type;
      record.arg = arg;

      if (finallyEntry) {
        this.method = "next";
        this.next = finallyEntry.finallyLoc;
        return ContinueSentinel;
      }

      return this.complete(record);
    },

    complete: function(record, afterLoc) {
      if (record.type === "throw") {
        throw record.arg;
      }

      if (record.type === "break" ||
          record.type === "continue") {
        this.next = record.arg;
      } else if (record.type === "return") {
        this.rval = this.arg = record.arg;
        this.method = "return";
        this.next = "end";
      } else if (record.type === "normal" && afterLoc) {
        this.next = afterLoc;
      }

      return ContinueSentinel;
    },

    finish: function(finallyLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.finallyLoc === finallyLoc) {
          this.complete(entry.completion, entry.afterLoc);
          resetTryEntry(entry);
          return ContinueSentinel;
        }
      }
    },

    "catch": function(tryLoc) {
      for (var i = this.tryEntries.length - 1; i >= 0; --i) {
        var entry = this.tryEntries[i];
        if (entry.tryLoc === tryLoc) {
          var record = entry.completion;
          if (record.type === "throw") {
            var thrown = record.arg;
            resetTryEntry(entry);
          }
          return thrown;
        }
      }

      // The context.catch method must only be called with a location
      // argument that corresponds to a known catch block.
      throw new Error("illegal catch attempt");
    },

    delegateYield: function(iterable, resultName, nextLoc) {
      this.delegate = {
        iterator: values(iterable),
        resultName: resultName,
        nextLoc: nextLoc
      };

      if (this.method === "next") {
        // Deliberately forget the last sent value so that we don't
        // accidentally pass it on to the delegate.
        this.arg = undefined$1;
      }

      return ContinueSentinel;
    }
  };

  // Regardless of whether this script is executing as a CommonJS module
  // or not, return the runtime object so that we can declare the variable
  // regeneratorRuntime in the outer scope, which allows this module to be
  // injected easily by `bin/regenerator --include-runtime script.js`.
  return exports;

}(
  // If this script is executing as a CommonJS module, use module.exports
  // as the regeneratorRuntime namespace. Otherwise create a new empty
  // object. Either way, the resulting object will be used to initialize
  // the regeneratorRuntime variable at the top of this file.
   module.exports 
));

try {
  regeneratorRuntime = runtime;
} catch (accidentalStrictMode) {
  // This module should not be running in strict mode, so the above
  // assignment should always work unless something is misconfigured. Just
  // in case runtime.js accidentally runs in strict mode, we can escape
  // strict mode using a global Function call. This could conceivably fail
  // if a Content Security Policy forbids using Function, but in that case
  // the proper solution is to fix the accidental strict mode problem. If
  // you've misconfigured your bundler to force strict mode and applied a
  // CSP to forbid Function, and you're not willing to fix either of those
  // problems, please detail your unique predicament in a GitHub issue.
  Function("r", "regeneratorRuntime = r")(runtime);
}
});

var PROD_PROGRAM_ID = 'boxndjnzQZEWbBku3YipL4pchYRc1zi4nNSrFUwawWo';
var DEV_PROGRAM_ID = 'dbo6b6DDmWTWpLcZM5SoVEf9X8DSJYPjVfD3Ptz6hod';
var STAGE = process.env.REACT_APP_STAGE || process.env.NODE_ENV;
var PROGRAM_ID = STAGE === 'development' ? /*#__PURE__*/new PublicKey(DEV_PROGRAM_ID) : /*#__PURE__*/new PublicKey(PROD_PROGRAM_ID);
var SOLANA_COMMITMENT = 'single'; // for message channels, the focus is speed rather than safety
// must match get_cek_account_address_with_seed in state.rs

var CEK_ACCOUNT_NONCE_SEED_STRING = 'solarium_cek_account'; // must match get_channel_address_with_seed in state.rs

var CHANNEL_NONCE_SEED_STRING = 'solarium_channel';
var USER_DETAILS_ACCOUNT_NONCE_SEED_STRING = 'solarium_userdetails_account'; // The cluster reference for the sol-did client - used to generate
// DIDs that can be unambiguously resolved to any solana cluster
// e.g. did:sol:devnet:abcde

var DEFAULT_CLUSTER = /*#__PURE__*/ClusterType.development(); // localhost - override with process.env.CLUSTER

var MESSAGE_SIZE_BYTES = 512;
var DEFAULT_MAX_MESSAGE_COUNT = 8;
var DEFAULT_USER_DETAILS_SIZE = 512;

var debug = /*#__PURE__*/Debug('solarium-js');
var privateKeyIsArray = function privateKeyIsArray(privateKey) {
  return Array.isArray(privateKey);
};
var privateKeyIsString = function privateKeyIsString(privateKey) {
  return typeof privateKey === 'string';
};
var privateKeyIsBuffer = function privateKeyIsBuffer(privateKey) {
  return Buffer.isBuffer(privateKey);
};
var privateKeyIsUint8Array = function privateKeyIsUint8Array(privateKey) {
  return privateKey instanceof Uint8Array;
};
/**
 * Create a Solana keypair object from an x25519 private key
 * @param privateKey
 */

var makeKeypair = function makeKeypair(privateKey) {
  if (privateKeyIsArray(privateKey)) return Keypair.fromSecretKey(Buffer.from(privateKey));
  if (privateKeyIsUint8Array(privateKey) || privateKeyIsBuffer(privateKey)) return Keypair.fromSecretKey(privateKey);

  if (privateKeyIsString(privateKey)) {
    var privateKeyHex = decode(privateKey);
    return Keypair.fromSecretKey(privateKeyHex);
  }

  throw new Error('Incompatible private key format');
};
var getClusterEndpoint = function getClusterEndpoint(cluster) {
  if (!cluster) return currentCluster().solanaUrl();

  switch (cluster) {
    case 'localnet':
      return 'http://localhost:8899';

    default:
      return clusterApiUrl(cluster);
  }
};
var toSolanaKeyMaterial = function toSolanaKeyMaterial(k) {
  if (isKeypair(k) || isPublicKey(k)) return k;

  try {
    return makeKeypair(k);
  } catch (_unused) {
    return new PublicKey(k);
  }
};
var didToPublicKey = function didToPublicKey(did) {
  return DecentralizedIdentifier.parse(did).pubkey.toPublicKey();
};
var currentCluster = function currentCluster(cluster) {
  return cluster ? ClusterType.parse(cluster) : process.env.CLUSTER ? ClusterType.parse(process.env.CLUSTER) : DEFAULT_CLUSTER;
}; // Typescript does not allow you to pass a possibly undefined value into a spread parameter
// and have it behave as if it is not there,
// in other words, if you have ...a as a parameter, and you pass in (undefined), this function
// will assign a to []

var arrayOf = function arrayOf() {
  for (var _len = arguments.length, things = new Array(_len), _key = 0; _key < _len; _key++) {
    things[_key] = arguments[_key];
  }

  return things.filter(function (t) {
    return t !== undefined;
  });
};
var isKeypair = function isKeypair(k) {
  return k instanceof Keypair;
};
var isPublicKey = function isPublicKey(k) {
  return k instanceof PublicKey;
};
var pubkeyOf = function pubkeyOf(k) {
  return isKeypair(k) ? k.publicKey : k;
};

var memoizedGetConnection = /*#__PURE__*/memoizeWith(getClusterEndpoint, function (cluster) {
  return new Connection(getClusterEndpoint(cluster), SOLANA_COMMITMENT);
});
var SolanaUtil = /*#__PURE__*/function () {
  function SolanaUtil() {}

  SolanaUtil.sendAndConfirmRawTransaction = function sendAndConfirmRawTransaction$1(connection, transaction) {
    return sendAndConfirmRawTransaction(connection, transaction.serialize(), {
      skipPreflight: false,
      commitment: SOLANA_COMMITMENT,
      preflightCommitment: SOLANA_COMMITMENT
    });
  };

  SolanaUtil.newWalletWithLamports = /*#__PURE__*/function () {
    var _newWalletWithLamports = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(connection, lamports) {
      var keypair;
      return runtime_1.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (lamports === void 0) {
                lamports = 1000000;
              }

              keypair = Keypair.generate();
              _context.next = 4;
              return this.airdrop(connection, keypair.publicKey, lamports);

            case 4:
              return _context.abrupt("return", keypair);

            case 5:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function newWalletWithLamports(_x, _x2) {
      return _newWalletWithLamports.apply(this, arguments);
    }

    return newWalletWithLamports;
  }();

  SolanaUtil.airdrop = /*#__PURE__*/function () {
    var _airdrop = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(connection, publicKey, lamports) {
      var retries, balance;
      return runtime_1.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              if (lamports === void 0) {
                lamports = 1000000;
              }

              retries = 30;
              _context2.next = 4;
              return connection.requestAirdrop(publicKey, lamports);

            case 4:
              _context2.next = 6;
              return this.sleep(500);

            case 6:
              _context2.next = 8;
              return connection.getBalance(publicKey);

            case 8:
              balance = _context2.sent;

              if (!(lamports <= balance)) {
                _context2.next = 11;
                break;
              }

              return _context2.abrupt("return");

            case 11:
              if (!(--retries <= 0)) {
                _context2.next = 13;
                break;
              }

              return _context2.abrupt("break", 15);

            case 13:
              _context2.next = 4;
              break;

            case 15:
              throw new Error("Airdrop of " + lamports + " failed");

            case 16:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    function airdrop(_x3, _x4, _x5) {
      return _airdrop.apply(this, arguments);
    }

    return airdrop;
  }();

  SolanaUtil.sleep = function sleep(ms) {
    return new Promise(function (resolve) {
      return setTimeout(resolve, ms);
    });
  };

  return SolanaUtil;
}();
SolanaUtil.getConnection = memoizedGetConnection;
var airdrop = function airdrop(connection, publicKey, lamports) {
  if (lamports === void 0) {
    lamports = 1000000;
  }

  return SolanaUtil.airdrop(connection, publicKey, lamports);
};
var getConnection = function getConnection(cluster) {
  return SolanaUtil.getConnection(cluster);
};

function writeUint32BE(value, array) {
  if (array === void 0) {
    array = new Uint8Array(4);
  }

  var encoded = fromString(value.toString(), 'base10');
  array.set(encoded, 4 - encoded.length);
  return array;
}

var lengthAndInput = function lengthAndInput(input) {
  return concat([writeUint32BE(input.length), input]);
}; // Implementation from:
// https://github.com/decentralized-identity/did-jwt


function concatKDF(secret, keyLen, alg) {
  if (keyLen !== 256) throw new Error("Unsupported key length: " + keyLen);
  var value = concat([lengthAndInput(fromString(alg)), lengthAndInput(new Uint8Array(0)), lengthAndInput(new Uint8Array(0)), writeUint32BE(keyLen)]); // since our key lenght is 256 we only have to do one round

  var roundNumber = 1;
  return hash(concat([writeUint32BE(roundNumber), secret, value]));
}
var base64ToBytes = function base64ToBytes(s) {
  return fromString(s, 'base64pad');
};
var bytesToBase64 = function bytesToBase64(b) {
  return toString(b, 'base64pad');
};
var stringToBytes = function stringToBytes(s) {
  return fromString(s);
};
var bytesToString = function bytesToString(s) {
  return toString(s);
};
var base58ToBytes = function base58ToBytes(s) {
  return fromString(s, 'base58btc');
};
var bytesToBase58 = function bytesToBase58(b) {
  return toString(b, 'base58btc');
};

var Assignable = /*#__PURE__*/function () {
  function Assignable(properties) {
    var _this = this;

    Object.keys(properties).forEach(function (key) {
      _this[key] = properties[key];
    });
  }

  var _proto = Assignable.prototype;

  _proto.encode = function encode() {
    return Buffer.from(serialize(SCHEMA, this));
  };

  Assignable.decode = function decode(data) {
    return deserializeExtraBytes(SCHEMA, this, data);
  };

  return Assignable;
}(); // Class representing a Rust-compatible enum, since enums are only strings or
// numbers in pure JS

var Enum = /*#__PURE__*/function (_Assignable) {
  _inheritsLoose(Enum, _Assignable);

  function Enum(properties) {
    var _this2;

    _this2 = _Assignable.call(this, properties) || this;

    if (Object.keys(properties).length !== 1) {
      throw new Error('Enum can only take single value');
    }

    _this2["enum"] = '';
    Object.keys(properties).forEach(function (key) {
      _this2["enum"] = key;
    });
    return _this2;
  }

  return Enum;
}(Assignable);
var SCHEMA = /*#__PURE__*/new Map(); // TODO PR for leaving extra bytes, a lot of code copied from
// https://github.com/near/borsh-js/blob/master/borsh-ts/index.ts

var capitalizeFirstLetter = function capitalizeFirstLetter(s) {
  return s.charAt(0).toUpperCase() + s.slice(1);
};

function deserializeField(schema, fieldName, fieldType, reader) {
  try {
    if (typeof fieldType === 'string') {
      return reader["read" + capitalizeFirstLetter(fieldType)]();
    }

    if (fieldType instanceof Array) {
      if (typeof fieldType[0] === 'number') {
        return reader.readFixedArray(fieldType[0]);
      }

      return reader.readArray(function () {
        return deserializeField(schema, fieldName, fieldType[0], reader);
      });
    }

    return deserializeStruct(schema, fieldType, reader);
  } catch (error) {
    if (error instanceof BorshError) {
      error.addToFieldPath(fieldName);
    }

    throw error;
  }
}

function deserializeStruct(schema, classType, reader) {
  var structSchema = schema.get(classType);

  if (!structSchema) {
    throw new BorshError("Class " + classType.name + " is missing in schema");
  }

  if (structSchema.kind === 'struct') {
    var result = {};

    for (var _iterator = _createForOfIteratorHelperLoose(schema.get(classType).fields), _step; !(_step = _iterator()).done;) {
      var _step$value = _step.value,
          fieldName = _step$value[0],
          fieldType = _step$value[1];
      result[fieldName] = deserializeField(schema, fieldName, fieldType, reader);
    }

    return new classType(result);
  }

  if (structSchema.kind === 'enum') {
    var _classType;

    var idx = reader.readU8();

    if (idx >= structSchema.values.length) {
      throw new BorshError("Enum index: " + idx + " is out of range");
    }

    var _structSchema$values$ = structSchema.values[idx],
        _fieldName = _structSchema$values$[0],
        _fieldType = _structSchema$values$[1];
    var fieldValue = deserializeField(schema, _fieldName, _fieldType, reader);
    return new classType((_classType = {}, _classType[_fieldName] = fieldValue, _classType));
  }

  throw new BorshError("Unexpected schema kind: " + structSchema.kind + " for " + classType.constructor.name);
} /// Deserializes object from bytes using schema.


function deserializeExtraBytes(schema, classType, buffer) {
  var reader = new BinaryReader(buffer);
  return deserializeStruct(schema, classType, reader);
}
var AssignablePublicKey = /*#__PURE__*/function (_Assignable2) {
  _inheritsLoose(AssignablePublicKey, _Assignable2);

  function AssignablePublicKey() {
    return _Assignable2.apply(this, arguments) || this;
  }

  var _proto2 = AssignablePublicKey.prototype;

  _proto2.toPublicKey = function toPublicKey() {
    return new PublicKey(this.bytes);
  };

  _proto2.toString = function toString() {
    return bytesToBase58(Uint8Array.from(this.bytes));
  };

  AssignablePublicKey.parse = function parse(pubkey) {
    return AssignablePublicKey.fromPublicKey(new PublicKey(pubkey));
  };

  AssignablePublicKey.fromPublicKey = function fromPublicKey(publicKey) {
    return new AssignablePublicKey({
      bytes: Uint8Array.from(publicKey.toBuffer())
    });
  };

  AssignablePublicKey.empty = function empty() {
    var bytes = new Array(32);
    bytes.fill(0);
    return new AssignablePublicKey({
      bytes: bytes
    });
  };

  return AssignablePublicKey;
}(Assignable);
SCHEMA.set(AssignablePublicKey, {
  kind: 'struct',
  fields: [['bytes', [32]]]
});

var CEKData = /*#__PURE__*/function (_Assignable) {
  _inheritsLoose(CEKData, _Assignable);

  function CEKData() {
    return _Assignable.apply(this, arguments) || this;
  }

  return CEKData;
}(Assignable);
SCHEMA.set(CEKData, {
  kind: 'struct',
  fields: [['header', 'string'], ['kid', 'string'], ['encryptedKey', 'string']]
});

var InitializeChannel = /*#__PURE__*/function (_Assignable) {
  _inheritsLoose(InitializeChannel, _Assignable);

  function InitializeChannel() {
    return _Assignable.apply(this, arguments) || this;
  }

  return InitializeChannel;
}(Assignable);
var InitializeDirectChannel = /*#__PURE__*/function (_Assignable2) {
  _inheritsLoose(InitializeDirectChannel, _Assignable2);

  function InitializeDirectChannel() {
    return _Assignable2.apply(this, arguments) || this;
  }

  return InitializeDirectChannel;
}(Assignable);
var Post = /*#__PURE__*/function (_Assignable3) {
  _inheritsLoose(Post, _Assignable3);

  function Post() {
    return _Assignable3.apply(this, arguments) || this;
  }

  return Post;
}(Assignable);
var AddToChannel = /*#__PURE__*/function (_Assignable4) {
  _inheritsLoose(AddToChannel, _Assignable4);

  function AddToChannel() {
    return _Assignable4.apply(this, arguments) || this;
  }

  return AddToChannel;
}(Assignable);
var AddCEK = /*#__PURE__*/function (_Assignable5) {
  _inheritsLoose(AddCEK, _Assignable5);

  function AddCEK() {
    return _Assignable5.apply(this, arguments) || this;
  }

  return AddCEK;
}(Assignable);
var RemoveCEK = /*#__PURE__*/function (_Assignable6) {
  _inheritsLoose(RemoveCEK, _Assignable6);

  function RemoveCEK() {
    return _Assignable6.apply(this, arguments) || this;
  }

  return RemoveCEK;
}(Assignable);
var CreateUserDetails = /*#__PURE__*/function (_Assignable7) {
  _inheritsLoose(CreateUserDetails, _Assignable7);

  function CreateUserDetails() {
    return _Assignable7.apply(this, arguments) || this;
  }

  return CreateUserDetails;
}(Assignable);
var UpdateUserDetails = /*#__PURE__*/function (_Assignable8) {
  _inheritsLoose(UpdateUserDetails, _Assignable8);

  function UpdateUserDetails() {
    return _Assignable8.apply(this, arguments) || this;
  }

  return UpdateUserDetails;
}(Assignable);
var SolariumInstruction = /*#__PURE__*/function (_Enum) {
  _inheritsLoose(SolariumInstruction, _Enum);

  function SolariumInstruction() {
    return _Enum.apply(this, arguments) || this;
  }

  SolariumInstruction.initializeChannel = function initializeChannel(name, CEKs) {
    return new SolariumInstruction({
      initializeChannel: new InitializeChannel({
        name: name,
        CEKs: CEKs
      })
    });
  };

  SolariumInstruction.initializeDirectChannel = function initializeDirectChannel(creatorCEKs, inviteeCEKs) {
    return new SolariumInstruction({
      initializeDirectChannel: new InitializeDirectChannel({
        creatorCEKs: creatorCEKs,
        inviteeCEKs: inviteeCEKs
      })
    });
  };

  SolariumInstruction.post = function post(message) {
    return new SolariumInstruction({
      post: new Post({
        message: message
      })
    });
  };

  SolariumInstruction.addToChannel = function addToChannel(CEKs) {
    return new SolariumInstruction({
      addToChannel: new AddToChannel({
        CEKs: CEKs
      })
    });
  };

  SolariumInstruction.addCEK = function addCEK(cek) {
    return new SolariumInstruction({
      addCEK: new AddCEK({
        cek: cek
      })
    });
  };

  SolariumInstruction.removeCEK = function removeCEK(kid) {
    return new SolariumInstruction({
      removeCEK: new RemoveCEK({
        kid: kid
      })
    });
  };

  SolariumInstruction.createUserDetails = function createUserDetails(alias, addressBook, size) {
    if (addressBook === void 0) {
      addressBook = '';
    }

    if (size === void 0) {
      size = DEFAULT_USER_DETAILS_SIZE;
    }

    return new SolariumInstruction({
      createUserDetails: new CreateUserDetails({
        alias: alias,
        addressBook: addressBook,
        size: size
      })
    });
  };

  SolariumInstruction.updateUserDetails = function updateUserDetails(alias, addressBook) {
    if (addressBook === void 0) {
      addressBook = '';
    }

    return new SolariumInstruction({
      updateUserDetails: new UpdateUserDetails({
        alias: alias,
        addressBook: addressBook
      })
    });
  };

  return SolariumInstruction;
}(Enum);
function getCekAccountKey(_x, _x2) {
  return _getCekAccountKey.apply(this, arguments);
}

function _getCekAccountKey() {
  _getCekAccountKey = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(ownerDID, channel) {
    var publicKeyNonce;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return PublicKey.findProgramAddress([ownerDID.toBuffer(), channel.toBuffer(), Buffer.from(CEK_ACCOUNT_NONCE_SEED_STRING, 'utf8')], PROGRAM_ID);

          case 2:
            publicKeyNonce = _context.sent;
            return _context.abrupt("return", publicKeyNonce[0]);

          case 4:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _getCekAccountKey.apply(this, arguments);
}

function getUserDetailsKey(_x3) {
  return _getUserDetailsKey.apply(this, arguments);
}

function _getUserDetailsKey() {
  _getUserDetailsKey = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(did) {
    var publicKeyNonce;
    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return PublicKey.findProgramAddress([did.toBuffer(), Buffer.from(USER_DETAILS_ACCOUNT_NONCE_SEED_STRING, 'utf8')], PROGRAM_ID);

          case 2:
            publicKeyNonce = _context2.sent;
            return _context2.abrupt("return", publicKeyNonce[0]);

          case 4:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return _getUserDetailsKey.apply(this, arguments);
}

function getDirectChannelAccountKey(_x4, _x5) {
  return _getDirectChannelAccountKey.apply(this, arguments);
}

function _getDirectChannelAccountKey() {
  _getDirectChannelAccountKey = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee3(did1, did2) {
    var reverse, orderedDIDs, publicKeyNonce;
    return runtime_1.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            // To ensure predictable account addresses, sort the DIDs alphabetically.
            reverse = did1.toBuffer().compare(did2.toBuffer()) === 1;
            orderedDIDs = reverse ? [did2.toBuffer(), did1.toBuffer()] : [did1.toBuffer(), did2.toBuffer()];
            _context3.next = 4;
            return PublicKey.findProgramAddress([].concat(orderedDIDs, [Buffer.from(CHANNEL_NONCE_SEED_STRING, 'utf8')]), PROGRAM_ID);

          case 4:
            publicKeyNonce = _context3.sent;
            return _context3.abrupt("return", publicKeyNonce[0]);

          case 6:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));
  return _getDirectChannelAccountKey.apply(this, arguments);
}

function initializeChannel(_x6, _x7, _x8, _x9, _x10, _x11) {
  return _initializeChannel.apply(this, arguments);
}

function _initializeChannel() {
  _initializeChannel = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee4(payer, channel, name, creatorDID, creatorAuthority, CEKs) {
    var creatorCEKAccount, keys, data;
    return runtime_1.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return getCekAccountKey(creatorDID, channel);

          case 2:
            creatorCEKAccount = _context4.sent;
            keys = [{
              pubkey: payer,
              isSigner: true,
              isWritable: true
            }, {
              pubkey: channel,
              isSigner: false,
              isWritable: true
            }, {
              pubkey: creatorDID,
              isSigner: false,
              isWritable: false
            }, {
              pubkey: creatorAuthority,
              isSigner: true,
              isWritable: false
            }, {
              pubkey: creatorCEKAccount,
              isSigner: false,
              isWritable: true
            }, {
              pubkey: SYSVAR_RENT_PUBKEY,
              isSigner: false,
              isWritable: false
            }, {
              pubkey: SystemProgram.programId,
              isSigner: false,
              isWritable: false
            }];
            data = SolariumInstruction.initializeChannel(name, CEKs).encode();
            return _context4.abrupt("return", new TransactionInstruction({
              keys: keys,
              programId: PROGRAM_ID,
              data: data
            }));

          case 6:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));
  return _initializeChannel.apply(this, arguments);
}

function initializeDirectChannel(_x12, _x13, _x14, _x15, _x16, _x17, _x18) {
  return _initializeDirectChannel.apply(this, arguments);
}

function _initializeDirectChannel() {
  _initializeDirectChannel = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee5(payer, channel, creatorDID, creatorAuthority, inviteeDID, creatorCEKs, inviteeCEKs) {
    var creatorCEKAccount, inviteeCEKAccount, keys, data;
    return runtime_1.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return getCekAccountKey(creatorDID, channel);

          case 2:
            creatorCEKAccount = _context5.sent;
            _context5.next = 5;
            return getCekAccountKey(inviteeDID, channel);

          case 5:
            inviteeCEKAccount = _context5.sent;
            keys = [{
              pubkey: payer,
              isSigner: true,
              isWritable: true
            }, {
              pubkey: channel,
              isSigner: false,
              isWritable: true
            }, {
              pubkey: creatorDID,
              isSigner: false,
              isWritable: false
            }, {
              pubkey: creatorAuthority,
              isSigner: true,
              isWritable: false
            }, {
              pubkey: creatorCEKAccount,
              isSigner: false,
              isWritable: true
            }, {
              pubkey: inviteeDID,
              isSigner: false,
              isWritable: false
            }, {
              pubkey: inviteeCEKAccount,
              isSigner: false,
              isWritable: true
            }, {
              pubkey: SYSVAR_RENT_PUBKEY,
              isSigner: false,
              isWritable: false
            }, {
              pubkey: SystemProgram.programId,
              isSigner: false,
              isWritable: false
            }];
            data = SolariumInstruction.initializeDirectChannel(creatorCEKs, inviteeCEKs).encode();
            return _context5.abrupt("return", new TransactionInstruction({
              keys: keys,
              programId: PROGRAM_ID,
              data: data
            }));

          case 9:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5);
  }));
  return _initializeDirectChannel.apply(this, arguments);
}

function post(_x19, _x20, _x21) {
  return _post.apply(this, arguments);
}

function _post() {
  _post = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee6(channel, senderAuthority, message) {
    var senderCEKAccount, keys, data;
    return runtime_1.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            _context6.next = 2;
            return getCekAccountKey(message.sender.toPublicKey(), channel);

          case 2:
            senderCEKAccount = _context6.sent;
            keys = [{
              pubkey: channel,
              isSigner: false,
              isWritable: true
            }, {
              pubkey: message.sender.toPublicKey(),
              isSigner: false,
              isWritable: false
            }, {
              pubkey: senderAuthority,
              isSigner: true,
              isWritable: false
            }, {
              pubkey: senderCEKAccount,
              isSigner: false,
              isWritable: false
            }];
            data = SolariumInstruction.post(message.content).encode();
            return _context6.abrupt("return", new TransactionInstruction({
              keys: keys,
              programId: PROGRAM_ID,
              data: data
            }));

          case 6:
          case "end":
            return _context6.stop();
        }
      }
    }, _callee6);
  }));
  return _post.apply(this, arguments);
}

function addToChannel(_x22, _x23, _x24, _x25, _x26, _x27) {
  return _addToChannel.apply(this, arguments);
}

function _addToChannel() {
  _addToChannel = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee7(payer, channel, inviteeDID, inviterDID, inviterAuthority, CEKs) {
    var inviterCEKAccount, inviteeCEKAccount, keys, data;
    return runtime_1.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            _context7.next = 2;
            return getCekAccountKey(inviterDID, channel);

          case 2:
            inviterCEKAccount = _context7.sent;
            _context7.next = 5;
            return getCekAccountKey(inviteeDID, channel);

          case 5:
            inviteeCEKAccount = _context7.sent;
            keys = [{
              pubkey: payer,
              isSigner: true,
              isWritable: true
            }, {
              pubkey: inviteeDID,
              isSigner: false,
              isWritable: false
            }, {
              pubkey: inviterDID,
              isSigner: false,
              isWritable: false
            }, {
              pubkey: inviterAuthority,
              isSigner: true,
              isWritable: false
            }, {
              pubkey: inviterCEKAccount,
              isSigner: false,
              isWritable: false
            }, {
              pubkey: inviteeCEKAccount,
              isSigner: false,
              isWritable: true
            }, {
              pubkey: channel,
              isSigner: false,
              isWritable: true
            }, {
              pubkey: SYSVAR_RENT_PUBKEY,
              isSigner: false,
              isWritable: false
            }, {
              pubkey: SystemProgram.programId,
              isSigner: false,
              isWritable: false
            }];
            data = SolariumInstruction.addToChannel(CEKs).encode();
            return _context7.abrupt("return", new TransactionInstruction({
              keys: keys,
              programId: PROGRAM_ID,
              data: data
            }));

          case 9:
          case "end":
            return _context7.stop();
        }
      }
    }, _callee7);
  }));
  return _addToChannel.apply(this, arguments);
}

function addCEK(_x28, _x29, _x30, _x31) {
  return _addCEK.apply(this, arguments);
}

function _addCEK() {
  _addCEK = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee8(channel, ownerDID, ownerAuthority, cek) {
    var ownerCEKAccount, keys, data;
    return runtime_1.wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            _context8.next = 2;
            return getCekAccountKey(ownerDID, channel);

          case 2:
            ownerCEKAccount = _context8.sent;
            keys = [{
              pubkey: ownerDID,
              isSigner: false,
              isWritable: false
            }, {
              pubkey: ownerAuthority,
              isSigner: true,
              isWritable: false
            }, {
              pubkey: ownerCEKAccount,
              isSigner: false,
              isWritable: true
            }];
            data = SolariumInstruction.addCEK(cek).encode();
            return _context8.abrupt("return", new TransactionInstruction({
              keys: keys,
              programId: PROGRAM_ID,
              data: data
            }));

          case 6:
          case "end":
            return _context8.stop();
        }
      }
    }, _callee8);
  }));
  return _addCEK.apply(this, arguments);
}

function createUserDetails(_x36, _x37, _x38, _x39, _x40) {
  return _createUserDetails.apply(this, arguments);
}

function _createUserDetails() {
  _createUserDetails = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee10(payer, did, authority, alias, size) {
    var userDetailsAccount, keys, data;
    return runtime_1.wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            _context10.next = 2;
            return getUserDetailsKey(did);

          case 2:
            userDetailsAccount = _context10.sent;
            keys = [{
              pubkey: payer,
              isSigner: true,
              isWritable: true
            }, {
              pubkey: did,
              isSigner: false,
              isWritable: false
            }, {
              pubkey: authority,
              isSigner: true,
              isWritable: false
            }, {
              pubkey: userDetailsAccount,
              isSigner: false,
              isWritable: true
            }, {
              pubkey: SYSVAR_RENT_PUBKEY,
              isSigner: false,
              isWritable: false
            }, {
              pubkey: SystemProgram.programId,
              isSigner: false,
              isWritable: false
            }];
            data = SolariumInstruction.createUserDetails(alias, undefined, size).encode();
            return _context10.abrupt("return", new TransactionInstruction({
              keys: keys,
              programId: PROGRAM_ID,
              data: data
            }));

          case 6:
          case "end":
            return _context10.stop();
        }
      }
    }, _callee10);
  }));
  return _createUserDetails.apply(this, arguments);
}

function updateUserDetails(_x41, _x42, _x43, _x44) {
  return _updateUserDetails.apply(this, arguments);
}

function _updateUserDetails() {
  _updateUserDetails = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee11(did, authority, alias, addressBook) {
    var userDetailsAccount, keys, data;
    return runtime_1.wrap(function _callee11$(_context11) {
      while (1) {
        switch (_context11.prev = _context11.next) {
          case 0:
            _context11.next = 2;
            return getUserDetailsKey(did);

          case 2:
            userDetailsAccount = _context11.sent;
            keys = [{
              pubkey: did,
              isSigner: false,
              isWritable: false
            }, {
              pubkey: authority,
              isSigner: true,
              isWritable: false
            }, {
              pubkey: userDetailsAccount,
              isSigner: false,
              isWritable: true
            }];
            data = SolariumInstruction.updateUserDetails(alias, addressBook).encode();
            return _context11.abrupt("return", new TransactionInstruction({
              keys: keys,
              programId: PROGRAM_ID,
              data: data
            }));

          case 6:
          case "end":
            return _context11.stop();
        }
      }
    }, _callee11);
  }));
  return _updateUserDetails.apply(this, arguments);
}

SCHEMA.set(SolariumInstruction, {
  kind: 'enum',
  field: 'enum',
  values: [['initializeChannel', InitializeChannel], ['initializeDirectChannel', InitializeDirectChannel], ['post', Post], ['addToChannel', AddToChannel], ['addCEK', AddCEK], ['removeCEK', RemoveCEK], ['createUserDetails', CreateUserDetails], ['updateUserDetails', updateUserDetails]]
});
SCHEMA.set(InitializeChannel, {
  kind: 'struct',
  fields: [['name', 'string'], ['CEKs', [CEKData]]]
});
SCHEMA.set(InitializeDirectChannel, {
  kind: 'struct',
  fields: [['creatorCEKs', [CEKData]], ['inviteeCEKs', [CEKData]]]
});
SCHEMA.set(Post, {
  kind: 'struct',
  fields: [['message', 'string']]
});
SCHEMA.set(AddToChannel, {
  kind: 'struct',
  fields: [['CEKs', [CEKData]]]
});
SCHEMA.set(AddCEK, {
  kind: 'struct',
  fields: [['cek', CEKData]]
});
SCHEMA.set(RemoveCEK, {
  kind: 'struct',
  fields: [['kid', 'string']]
});
SCHEMA.set(CreateUserDetails, {
  kind: 'struct',
  fields: [['alias', 'string'], ['addressBook', 'string'], ['size', 'u32']]
});
SCHEMA.set(UpdateUserDetails, {
  kind: 'struct',
  fields: [['alias', 'string'], ['addressBook', 'string']]
});

var MessageData = /*#__PURE__*/function (_Assignable) {
  _inheritsLoose(MessageData, _Assignable);

  function MessageData() {
    return _Assignable.apply(this, arguments) || this;
  }

  MessageData["for"] = function _for(senderDID, content) {
    return new MessageData({
      timestamp: 0,
      sender: AssignablePublicKey.fromPublicKey(senderDID),
      content: content
    });
  };

  return MessageData;
}(Assignable);
SCHEMA.set(MessageData, {
  kind: 'struct',
  fields: [['timestamp', 'u64'], ['sender', AssignablePublicKey], ['content', 'string']]
});

var ChannelData = /*#__PURE__*/function (_Assignable) {
  _inheritsLoose(ChannelData, _Assignable);

  function ChannelData() {
    return _Assignable.apply(this, arguments) || this;
  }

  ChannelData.fromAccount = function fromAccount(accountData) {
    return ChannelData.decode(accountData);
  };

  ChannelData.empty = function empty(name) {
    return new ChannelData({
      name: name,
      messages: []
    });
  } // The size of a channel in bytes - must match size_bytes in state.rs
  ;

  ChannelData.sizeBytes = function sizeBytes() {
    return MESSAGE_SIZE_BYTES * DEFAULT_MAX_MESSAGE_COUNT + 64;
  };

  return ChannelData;
}(Assignable);
SCHEMA.set(ChannelData, {
  kind: 'struct',
  fields: [['name', 'string'], ['messages', [MessageData]]]
});

var CEKAccountData = /*#__PURE__*/function (_Assignable) {
  _inheritsLoose(CEKAccountData, _Assignable);

  function CEKAccountData() {
    return _Assignable.apply(this, arguments) || this;
  }

  CEKAccountData.fromAccount = function fromAccount(accountData) {
    return CEKAccountData.decode(accountData);
  };

  CEKAccountData.empty = function empty(ownerDID, channel) {
    return new CEKAccountData({
      ownerDID: ownerDID,
      channel: channel,
      ceks: []
    });
  };

  return CEKAccountData;
}(Assignable);
SCHEMA.set(CEKAccountData, {
  kind: 'struct',
  fields: [['ownerDID', AssignablePublicKey], ['channel', AssignablePublicKey], ['ceks', [CEKData]]]
});

var UserDetailsData = /*#__PURE__*/function (_Assignable) {
  _inheritsLoose(UserDetailsData, _Assignable);

  function UserDetailsData() {
    return _Assignable.apply(this, arguments) || this;
  }

  UserDetailsData.fromAccount = function fromAccount(accountData) {
    return UserDetailsData.decode(accountData);
  };

  UserDetailsData.empty = function empty(alias) {
    return new UserDetailsData({
      alias: alias,
      addressBook: ''
    });
  };

  return UserDetailsData;
}(Assignable);
SCHEMA.set(UserDetailsData, {
  kind: 'struct',
  fields: [['alias', 'string'], ['addressBook', 'string']]
});

var SolariumTransaction = /*#__PURE__*/function () {
  function SolariumTransaction() {}

  SolariumTransaction.createGroupChannel = /*#__PURE__*/function () {
    var _createGroupChannel = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(connection, payer, creatorDID, creatorAuthority, name, initialCEKs, signCallback, cluster) {
      var channel, size, balanceNeeded, createChannelAccountInstruction, initializeChannelInstruction;
      return runtime_1.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              channel = Keypair.generate();
              debug("Channel address: " + channel.publicKey.toBase58());
              size = ChannelData.sizeBytes();
              _context.next = 5;
              return connection.getMinimumBalanceForRentExemption(size);

            case 5:
              balanceNeeded = _context.sent;
              createChannelAccountInstruction = SystemProgram.createAccount({
                programId: PROGRAM_ID,
                fromPubkey: payer,
                lamports: balanceNeeded,
                newAccountPubkey: channel.publicKey,
                space: size
              });
              _context.next = 9;
              return initializeChannel(payer, channel.publicKey, name, creatorDID, creatorAuthority, initialCEKs);

            case 9:
              initializeChannelInstruction = _context.sent;
              _context.next = 12;
              return SolariumTransaction.signAndSendTransaction([createChannelAccountInstruction, initializeChannelInstruction], signCallback, [channel], cluster);

            case 12:
              return _context.abrupt("return", channel.publicKey);

            case 13:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    function createGroupChannel(_x, _x2, _x3, _x4, _x5, _x6, _x7, _x8) {
      return _createGroupChannel.apply(this, arguments);
    }

    return createGroupChannel;
  }();

  SolariumTransaction.createDirectChannel = /*#__PURE__*/function () {
    var _createDirectChannel = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(payer, creatorDID, creatorAuthority, inviteeDID, creatorCEKs, inviteeCEKs, signCallback, cluster) {
      var channel, initializeDirectChannelInstruction;
      return runtime_1.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return getDirectChannelAccountKey(creatorDID, inviteeDID);

            case 2:
              channel = _context2.sent;
              debug("Channel address: " + channel.toBase58());
              _context2.next = 6;
              return initializeDirectChannel(payer, channel, creatorDID, creatorAuthority, inviteeDID, creatorCEKs, inviteeCEKs);

            case 6:
              initializeDirectChannelInstruction = _context2.sent;
              _context2.next = 9;
              return SolariumTransaction.signAndSendTransaction([initializeDirectChannelInstruction], signCallback, [], cluster);

            case 9:
              return _context2.abrupt("return", channel);

            case 10:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));

    function createDirectChannel(_x9, _x10, _x11, _x12, _x13, _x14, _x15, _x16) {
      return _createDirectChannel.apply(this, arguments);
    }

    return createDirectChannel;
  }();

  SolariumTransaction.addDIDToChannel = /*#__PURE__*/function () {
    var _addDIDToChannel = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee3(payer, channel, inviterDID, inviterAuthority, inviteeDID, ceks, signCallback, cluster) {
      var addToChannelInstruction;
      return runtime_1.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return addToChannel(payer, channel, inviteeDID, inviterDID, inviterAuthority, ceks);

            case 2:
              addToChannelInstruction = _context3.sent;
              _context3.next = 5;
              return SolariumTransaction.signAndSendTransaction([addToChannelInstruction], signCallback, [], cluster);

            case 5:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3);
    }));

    function addDIDToChannel(_x17, _x18, _x19, _x20, _x21, _x22, _x23, _x24) {
      return _addDIDToChannel.apply(this, arguments);
    }

    return addDIDToChannel;
  }();

  SolariumTransaction.getChannelData = /*#__PURE__*/function () {
    var _getChannelData = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee4(connection, channel) {
      var accountInfo;
      return runtime_1.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              _context4.next = 2;
              return connection.getAccountInfo(channel);

            case 2:
              accountInfo = _context4.sent;

              if (accountInfo) {
                _context4.next = 5;
                break;
              }

              return _context4.abrupt("return", null);

            case 5:
              return _context4.abrupt("return", ChannelData.fromAccount(accountInfo.data));

            case 6:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4);
    }));

    function getChannelData(_x25, _x26) {
      return _getChannelData.apply(this, arguments);
    }

    return getChannelData;
  }();

  SolariumTransaction.getCEKAccountData = /*#__PURE__*/function () {
    var _getCEKAccountData = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee5(connection, ownerDID, channel) {
      var cekAccount, accountInfo;
      return runtime_1.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              _context5.next = 2;
              return getCekAccountKey(ownerDID, channel);

            case 2:
              cekAccount = _context5.sent;
              _context5.next = 5;
              return connection.getAccountInfo(cekAccount);

            case 5:
              accountInfo = _context5.sent;

              if (accountInfo) {
                _context5.next = 8;
                break;
              }

              return _context5.abrupt("return", null);

            case 8:
              return _context5.abrupt("return", CEKAccountData.fromAccount(accountInfo.data));

            case 9:
            case "end":
              return _context5.stop();
          }
        }
      }, _callee5);
    }));

    function getCEKAccountData(_x27, _x28, _x29) {
      return _getCEKAccountData.apply(this, arguments);
    }

    return getCEKAccountData;
  }();

  SolariumTransaction.postMessage = /*#__PURE__*/function () {
    var _postMessage = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee6(channel, senderDID, senderAuthority, message, signCallback, cluster) {
      var messageData, instruction;
      return runtime_1.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              messageData = MessageData["for"](senderDID, message);
              _context6.next = 3;
              return post(channel, senderAuthority, messageData);

            case 3:
              instruction = _context6.sent;
              return _context6.abrupt("return", SolariumTransaction.signAndSendTransaction([instruction], signCallback, [], cluster));

            case 5:
            case "end":
              return _context6.stop();
          }
        }
      }, _callee6);
    }));

    function postMessage(_x30, _x31, _x32, _x33, _x34, _x35) {
      return _postMessage.apply(this, arguments);
    }

    return postMessage;
  }();

  SolariumTransaction.addCEKToAccount = /*#__PURE__*/function () {
    var _addCEKToAccount = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee7(channel, memberDID, memberAuthority, cek, signCallback, cluster) {
      var addToChannelInstruction;
      return runtime_1.wrap(function _callee7$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              _context7.next = 2;
              return addCEK(channel, memberDID, memberAuthority, cek);

            case 2:
              addToChannelInstruction = _context7.sent;
              _context7.next = 5;
              return SolariumTransaction.signAndSendTransaction([addToChannelInstruction], signCallback, [], cluster);

            case 5:
            case "end":
              return _context7.stop();
          }
        }
      }, _callee7);
    }));

    function addCEKToAccount(_x36, _x37, _x38, _x39, _x40, _x41) {
      return _addCEKToAccount.apply(this, arguments);
    }

    return addCEKToAccount;
  }();

  SolariumTransaction.createUserDetails = /*#__PURE__*/function () {
    var _createUserDetails2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee8(payer, did, authority, signCallback, alias, size, cluster) {
      var userDetails, createUserDetailsInstruction;
      return runtime_1.wrap(function _callee8$(_context8) {
        while (1) {
          switch (_context8.prev = _context8.next) {
            case 0:
              _context8.next = 2;
              return getUserDetailsKey(did);

            case 2:
              userDetails = _context8.sent;
              debug("userDetails address: " + userDetails.toBase58());
              _context8.next = 6;
              return createUserDetails(payer, did, authority, alias, size);

            case 6:
              createUserDetailsInstruction = _context8.sent;
              _context8.next = 9;
              return SolariumTransaction.signAndSendTransaction([createUserDetailsInstruction], signCallback, [], cluster);

            case 9:
              return _context8.abrupt("return", userDetails);

            case 10:
            case "end":
              return _context8.stop();
          }
        }
      }, _callee8);
    }));

    function createUserDetails$1(_x42, _x43, _x44, _x45, _x46, _x47, _x48) {
      return _createUserDetails2.apply(this, arguments);
    }

    return createUserDetails$1;
  }();

  SolariumTransaction.updateUserDetails = /*#__PURE__*/function () {
    var _updateUserDetails2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee9(did, authority, signCallback, alias, addressBook, cluster) {
      var userDetails, updateUserDetailsInstruction;
      return runtime_1.wrap(function _callee9$(_context9) {
        while (1) {
          switch (_context9.prev = _context9.next) {
            case 0:
              _context9.next = 2;
              return getUserDetailsKey(did);

            case 2:
              userDetails = _context9.sent;
              debug("userDetails address: " + userDetails.toBase58());
              _context9.next = 6;
              return updateUserDetails(did, authority, alias, addressBook);

            case 6:
              updateUserDetailsInstruction = _context9.sent;
              _context9.next = 9;
              return SolariumTransaction.signAndSendTransaction([updateUserDetailsInstruction], signCallback, [], cluster);

            case 9:
              return _context9.abrupt("return", userDetails);

            case 10:
            case "end":
              return _context9.stop();
          }
        }
      }, _callee9);
    }));

    function updateUserDetails$1(_x49, _x50, _x51, _x52, _x53, _x54) {
      return _updateUserDetails2.apply(this, arguments);
    }

    return updateUserDetails$1;
  }();

  SolariumTransaction.getUserDetails = /*#__PURE__*/function () {
    var _getUserDetails = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee10(connection, did) {
      var userDetails, accountInfo;
      return runtime_1.wrap(function _callee10$(_context10) {
        while (1) {
          switch (_context10.prev = _context10.next) {
            case 0:
              _context10.next = 2;
              return getUserDetailsKey(did);

            case 2:
              userDetails = _context10.sent;
              _context10.next = 5;
              return connection.getAccountInfo(userDetails);

            case 5:
              accountInfo = _context10.sent;

              if (accountInfo) {
                _context10.next = 8;
                break;
              }

              return _context10.abrupt("return", null);

            case 8:
              return _context10.abrupt("return", UserDetailsData.fromAccount(accountInfo.data));

            case 9:
            case "end":
              return _context10.stop();
          }
        }
      }, _callee10);
    }));

    function getUserDetails(_x55, _x56) {
      return _getUserDetails.apply(this, arguments);
    }

    return getUserDetails;
  }();

  SolariumTransaction.signAndSendTransaction = /*#__PURE__*/function () {
    var _signAndSendTransaction = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee11(instructions, createSignedTx, additionalSigners, cluster) {
      var connection, _yield$connection$get, recentBlockhash, transaction;

      return runtime_1.wrap(function _callee11$(_context11) {
        while (1) {
          switch (_context11.prev = _context11.next) {
            case 0:
              connection = SolanaUtil.getConnection(cluster);
              _context11.next = 3;
              return connection.getRecentBlockhash();

            case 3:
              _yield$connection$get = _context11.sent;
              recentBlockhash = _yield$connection$get.blockhash;
              _context11.next = 7;
              return createSignedTx(instructions, {
                recentBlockhash: recentBlockhash
              }, additionalSigners);

            case 7:
              transaction = _context11.sent;
              return _context11.abrupt("return", SolanaUtil.sendAndConfirmRawTransaction(connection, transaction));

            case 9:
            case "end":
              return _context11.stop();
          }
        }
      }, _callee11);
    }));

    function signAndSendTransaction(_x57, _x58, _x59, _x60) {
      return _signAndSendTransaction.apply(this, arguments);
    }

    return signAndSendTransaction;
  }();

  return SolariumTransaction;
}();

var DEFAULT_CACHE_TTL_MS = 1000 * 60 * 5; // 5 minutes

var SolariumCache = /*#__PURE__*/function () {
  function SolariumCache(wrappedFunction, ttl) {
    if (ttl === void 0) {
      ttl = DEFAULT_CACHE_TTL_MS;
    }

    this.wrappedFunction = wrappedFunction;
    this.ttl = ttl;
    this.store = {};
  }

  var _proto = SolariumCache.prototype;

  _proto.getCached = function getCached(key) {
    var _this$store$key;

    return (_this$store$key = this.store[key]) == null ? void 0 : _this$store$key.value;
  };

  _proto.set = function set(key, value) {
    this.store[key] = {
      expiry: Date.now() + this.ttl,
      value: value
    };
  };

  _proto.evictCache = function evictCache(key) {
    delete this.store[key];
  };

  _proto.load = /*#__PURE__*/function () {
    var _load = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(key, skipCache) {
      var entry,
          _len,
          args,
          _key,
          loadedValue,
          _args = arguments;

      return runtime_1.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              entry = this.store[key];

              if (!(!skipCache && entry && entry.expiry > Date.now())) {
                _context.next = 3;
                break;
              }

              return _context.abrupt("return", entry.value);

            case 3:
              for (_len = _args.length, args = /*#__PURE__*/new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
                args[_key - 2] = _args[_key];
              }

              _context.next = 6;
              return this.wrappedFunction.apply(this, [key].concat(args));

            case 6:
              loadedValue = _context.sent;
              if (loadedValue) this.set(key, loadedValue);
              return _context.abrupt("return", loadedValue);

            case 9:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function load(_x, _x2) {
      return _load.apply(this, arguments);
    }

    return load;
  }();

  return SolariumCache;
}();

var get = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(authority, cluster) {
    var didForAuthority;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return keyToIdentifier(authority, currentCluster(cluster));

          case 2:
            didForAuthority = _context.sent;
            return _context.abrupt("return", getDocument(didForAuthority));

          case 4:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function get(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();
var didCache = /*#__PURE__*/new SolariumCache(resolve);
var getDocument = /*#__PURE__*/function () {
  var _ref2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(did, skipCache) {
    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (skipCache === void 0) {
              skipCache = false;
            }

            return _context2.abrupt("return", didCache.load(did, skipCache));

          case 2:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function getDocument(_x3, _x4) {
    return _ref2.apply(this, arguments);
  };
}();

var create = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee() {
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (!(currentCluster() !== ClusterType.mainnetBeta())) {
              _context.next = 2;
              break;
            }

            return _context.abrupt("return", SolanaUtil.newWalletWithLamports(SolanaUtil.getConnection(), 10000000000));

          case 2:
            return _context.abrupt("return", Keypair.generate());

          case 3:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function create() {
    return _ref.apply(this, arguments);
  };
}();
var defaultSignCallback = function defaultSignCallback(payer) {
  for (var _len = arguments.length, signers = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    signers[_key - 1] = arguments[_key];
  }

  return /*#__PURE__*/function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(instructions, transactionOpts, additionalSigners) {
      var _Transaction;

      var comparator, _uniqWith, uniqueSigners, signerPubkeys, transaction, message, myAccountSignature;

      return runtime_1.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              if (additionalSigners === void 0) {
                additionalSigners = [];
              }

              // Remove any duplicates from the list of signers (including the payer)
              // Prefers the first item if two items compare equal based on the predicate.
              comparator = function comparator(a, b) {
                return a.publicKey.equals(b.publicKey);
              };

              _uniqWith = uniqWith(comparator, [payer].concat(signers)), uniqueSigners = _uniqWith.slice(1);
              signerPubkeys = uniqueSigners.map(function (s) {
                return {
                  signature: null,
                  publicKey: s.publicKey
                };
              });
              transaction = (_Transaction = new Transaction(_extends({}, transactionOpts, {
                signatures: signerPubkeys,
                feePayer: payer.publicKey
              }))).add.apply(_Transaction, instructions);

              if (uniqueSigners.length + additionalSigners.length) {
                transaction.partialSign.apply(transaction, uniqueSigners.concat(additionalSigners));
              }

              message = transaction.serializeMessage();
              myAccountSignature = nacl.sign.detached(message, payer.secretKey);
              transaction.addSignature(payer.publicKey, Buffer.from(myAccountSignature));
              return _context2.abrupt("return", transaction);

            case 10:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));

    return function (_x, _x2, _x3) {
      return _ref2.apply(this, arguments);
    };
  }();
};
var defaultSignCallbackFor = function defaultSignCallbackFor(payer) {
  for (var _len2 = arguments.length, potentialSigners = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    potentialSigners[_key2 - 1] = arguments[_key2];
  }

  var signers = potentialSigners.filter(function (s) {
    return isKeypair(s);
  });
  return defaultSignCallback.apply(void 0, [payer].concat(signers));
};

var create$1 = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(owner, payer, alias, signCallback, cluster) {
    var createSignedTx, _yield$createRegister, registerInstruction, didKey, instructions, didForAuthority, _didKey, createUserDetailsInstruction, did;

    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            createSignedTx = signCallback || isKeypair(payer) && defaultSignCallbackFor(payer, owner);

            if (createSignedTx) {
              _context.next = 3;
              break;
            }

            throw new Error('No payer or sign callback specified');

          case 3:
            _context.next = 5;
            return createRegisterInstruction({
              payer: pubkeyOf(payer),
              authority: pubkeyOf(owner)
            });

          case 5:
            _yield$createRegister = _context.sent;
            registerInstruction = _yield$createRegister[0];
            didKey = _yield$createRegister[1];
            instructions = [registerInstruction];

            if (!alias) {
              _context.next = 19;
              break;
            }

            _context.next = 12;
            return keyToIdentifier(pubkeyOf(owner), currentCluster(cluster));

          case 12:
            didForAuthority = _context.sent;
            _didKey = didToPublicKey(didForAuthority);
            debug('Creating user-details for the new DID: ' + didForAuthority);
            _context.next = 17;
            return createUserDetails(pubkeyOf(payer), _didKey, pubkeyOf(owner), alias);

          case 17:
            createUserDetailsInstruction = _context.sent;
            instructions.push(createUserDetailsInstruction);

          case 19:
            _context.next = 21;
            return SolariumTransaction.signAndSendTransaction(instructions, createSignedTx, [], cluster);

          case 21:
            did = DecentralizedIdentifier.create(didKey, currentCluster(cluster)).toString();
            return _context.abrupt("return", getDocument(did, true));

          case 23:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function create(_x, _x2, _x3, _x4, _x5) {
    return _ref.apply(this, arguments);
  };
}();

var XC20P_IV_LENGTH = NONCE_LENGTH;
var XC20P_TAG_LENGTH = TAG_LENGTH; // XC20P

var xc20pEncrypter = function xc20pEncrypter(key) {
  var cipher = new XChaCha20Poly1305(key);
  return function (cleartext, aad) {
    var iv = randomBytes(XC20P_IV_LENGTH);
    var sealed = cipher.seal(iv, cleartext, aad);
    return {
      ciphertext: sealed.subarray(0, sealed.length - XC20P_TAG_LENGTH),
      tag: sealed.subarray(sealed.length - XC20P_TAG_LENGTH),
      iv: iv
    };
  };
}; // XC20P

var xc20pDecrypter = function xc20pDecrypter(key) {
  var cipher = new XChaCha20Poly1305(key);
  return function (ciphertext, tag, iv, aad) {
    return cipher.open(iv, concat([ciphertext, tag]), aad);
  };
}; // KW ECDH-ES+XC20PKW

var ECDH_ES_XC20PKW_ALG = 'ECDH-ES+XC20PKW';
var ECDH_ES_XC20PKW_KEYLEN = 256;
var x25519xc20pKeyWrap = function x25519xc20pKeyWrap(publicKey) {
  return function (wrappedKey) {
    var epk = generateKeyPair();
    var sharedSecret = sharedKey(epk.secretKey, publicKey);
    var kek = concatKDF(sharedSecret, ECDH_ES_XC20PKW_KEYLEN, ECDH_ES_XC20PKW_ALG);
    var res = xc20pEncrypter(kek)(wrappedKey);
    return {
      encryptedKey: res.ciphertext,
      tag: res.tag,
      iv: res.iv,
      epPubKey: epk.publicKey
    };
  };
};
var x25519xc20pKeyUnwrap = function x25519xc20pKeyUnwrap(secretKey) {
  return function (ciphertext, tag, iv, epPubKey) {
    var sharedSecret = sharedKey(secretKey, epPubKey); // Key Encryption Key

    var kek = concatKDF(sharedSecret, ECDH_ES_XC20PKW_KEYLEN, ECDH_ES_XC20PKW_ALG); // Content Encryption Key

    return xc20pDecrypter(kek)(ciphertext, tag, iv);
  };
};

var shortenKID = function shortenKID(kid) {
  return kid.substring(kid.indexOf('#') + 1);
}; // Create a CEK for a new channel


var generateCEK = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee() {
    var cek;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            cek = randomBytes(32);
            return _context.abrupt("return", Promise.resolve(cek));

          case 2:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function generateCEK() {
    return _ref.apply(this, arguments);
  };
}(); // given a key or reference to a key in a DID, return the key itself

var getVerificationMethod = function getVerificationMethod(vmOrRef, document) {
  if (Object.prototype.hasOwnProperty.call(vmOrRef, 'id')) return vmOrRef;
  var foundKey = (document.verificationMethod || []).find(function (key) {
    return key.id === vmOrRef;
  });
  if (!foundKey) throw new Error("Missing key " + vmOrRef);
  return foundKey;
}; // Given an unecrypted channel CEK, encrypt it for a DID


var encryptCEKForDID = /*#__PURE__*/function () {
  var _ref2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(cek, did) {
    var didDocument, augmentedDIDDocument;
    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return getDocument(did);

          case 2:
            didDocument = _context2.sent;
            augmentedDIDDocument = augmentDIDDocument(didDocument);
            return _context2.abrupt("return", encryptCEKForDIDDocument(cek, augmentedDIDDocument));

          case 5:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function encryptCEKForDID(_x, _x2) {
    return _ref2.apply(this, arguments);
  };
}();
var encryptCEKForVerificationMethod = /*#__PURE__*/function () {
  var _ref3 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee3(cek, key) {
    var res, concatByteArray, header;
    return runtime_1.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            if (key.publicKeyBase58) {
              _context3.next = 2;
              break;
            }

            throw Error('Currently we expect the recipient key to be encoded as base58');

          case 2:
            _context3.next = 4;
            return x25519xc20pKeyWrap(base58ToBytes(key.publicKeyBase58))(cek);

          case 4:
            res = _context3.sent;
            // header.iv + header.tag + header.epk.x
            concatByteArray = concat([res.iv, res.tag, res.epPubKey]);
            header = bytesToBase64(concatByteArray);
            return _context3.abrupt("return", new CEKData({
              kid: shortenKID(key.id),
              header: header,
              encryptedKey: bytesToBase64(res.encryptedKey)
            }));

          case 8:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function encryptCEKForVerificationMethod(_x3, _x4) {
    return _ref3.apply(this, arguments);
  };
}(); // Given an unencrypted channel CEK, encrypt it for a DID Document

var encryptCEKForDIDDocument = /*#__PURE__*/function () {
  var _ref4 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee5(cek, didDocument) {
    var encryptedCEKPromises;
    return runtime_1.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            encryptedCEKPromises = (didDocument.keyAgreement || []).map( /*#__PURE__*/function () {
              var _ref5 = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee4(keyOrRef) {
                var verificationMethod;
                return runtime_1.wrap(function _callee4$(_context4) {
                  while (1) {
                    switch (_context4.prev = _context4.next) {
                      case 0:
                        verificationMethod = getVerificationMethod(keyOrRef, didDocument);
                        return _context4.abrupt("return", encryptCEKForVerificationMethod(cek, verificationMethod));

                      case 2:
                      case "end":
                        return _context4.stop();
                    }
                  }
                }, _callee4);
              }));

              return function (_x7) {
                return _ref5.apply(this, arguments);
              };
            }());
            return _context5.abrupt("return", Promise.all(encryptedCEKPromises));

          case 2:
          case "end":
            return _context5.stop();
        }
      }
    }, _callee5);
  }));

  return function encryptCEKForDIDDocument(_x5, _x6) {
    return _ref4.apply(this, arguments);
  };
}(); // Create a new CEK and encrypt it for the DID

var createEncryptedCEK = /*#__PURE__*/function () {
  var _ref6 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee6(did) {
    var cek;
    return runtime_1.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            _context6.next = 2;
            return generateCEK();

          case 2:
            cek = _context6.sent;
            return _context6.abrupt("return", encryptCEKForDID(cek, did));

          case 4:
          case "end":
            return _context6.stop();
        }
      }
    }, _callee6);
  }));

  return function createEncryptedCEK(_x8) {
    return _ref6.apply(this, arguments);
  };
}(); // Decrypt an encrypted CEK for the with the key that was used to encrypt it

var decryptCEK = /*#__PURE__*/function () {
  var _ref7 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee7(encryptedCEK, key) {
    var encodedHeader, iv, tag, epkPub, encryptedKey, ed25519Key, curve25519Key, cek;
    return runtime_1.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            // decode information from CEKData
            encodedHeader = base64ToBytes(encryptedCEK.header); // iv (24), tag (16), epk PubKey (rest)

            iv = encodedHeader.subarray(0, XC20P_IV_LENGTH);
            tag = encodedHeader.subarray(XC20P_IV_LENGTH, XC20P_IV_LENGTH + XC20P_TAG_LENGTH);
            epkPub = encodedHeader.subarray(XC20P_IV_LENGTH + XC20P_TAG_LENGTH);
            encryptedKey = base64ToBytes(encryptedCEK.encryptedKey); // normalise the key into an uint array

            ed25519Key = makeKeypair(key).secretKey; // The key is used both for Ed25519 signing and x25519 ECDH encryption
            // the two different protocols use the same curve (Curve25519) but
            // different key formats. Specifically Ed25519 uses a 64 byte secret key
            // (which is the same format used by Solana), which is in fact a keypair
            // i.e. combination of the secret and public key, whereas x25519 uses a 32 byte
            // secret key. In order to use the same key for both, we convert here
            // from Ed25519 to x25519 format.

            curve25519Key = convertSecretKey(ed25519Key);
            _context7.next = 9;
            return x25519xc20pKeyUnwrap(curve25519Key)(encryptedKey, tag, iv, epkPub);

          case 9:
            cek = _context7.sent;

            if (!(cek === null)) {
              _context7.next = 12;
              break;
            }

            throw Error('There was a problem decrypting the CEK');

          case 12:
            return _context7.abrupt("return", cek);

          case 13:
          case "end":
            return _context7.stop();
        }
      }
    }, _callee7);
  }));

  return function decryptCEK(_x9, _x10) {
    return _ref7.apply(this, arguments);
  };
}(); // Find the CEK encrypted with a particular key, and decrypt it

var decryptCEKs = /*#__PURE__*/function () {
  var _ref8 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee8(encryptedCEKs, kid, key) {
    var encryptedCEK;
    return runtime_1.wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            // find the encrypted CEK for the key
            encryptedCEK = encryptedCEKs.find(function (k) {
              return k.kid === shortenKID(kid) || k.kid === kid;
            });

            if (encryptedCEK) {
              _context8.next = 3;
              break;
            }

            throw new Error("No encrypted CEK found for key " + kid);

          case 3:
            return _context8.abrupt("return", decryptCEK(encryptedCEK, key));

          case 4:
          case "end":
            return _context8.stop();
        }
      }
    }, _callee8);
  }));

  return function decryptCEKs(_x11, _x12, _x13) {
    return _ref8.apply(this, arguments);
  };
}(); // Encrypt a message with a CEK

var encryptMessage = /*#__PURE__*/function () {
  var _ref9 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee9(message, cek) {
    var encryptMessage, encodedEncMessage;
    return runtime_1.wrap(function _callee9$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            _context9.next = 2;
            return xc20pEncrypter(cek)(stringToBytes(message));

          case 2:
            encryptMessage = _context9.sent;
            // iv (24), ciphertext (var), tag (16)
            encodedEncMessage = bytesToBase64(concat([encryptMessage.iv, encryptMessage.ciphertext, encryptMessage.tag]));
            return _context9.abrupt("return", encodedEncMessage);

          case 5:
          case "end":
            return _context9.stop();
        }
      }
    }, _callee9);
  }));

  return function encryptMessage(_x14, _x15) {
    return _ref9.apply(this, arguments);
  };
}(); // Decrypt a message with the CEK used to encrypt it

var decryptMessage = /*#__PURE__*/function () {
  var _ref10 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee10(encryptedMessage, cek) {
    var encMessage, iv, ciphertext, tag, binMessage, message;
    return runtime_1.wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            encMessage = base64ToBytes(encryptedMessage); // iv (24), ciphertext (var), tag (16)

            iv = encMessage.subarray(0, XC20P_IV_LENGTH);
            ciphertext = encMessage.subarray(XC20P_IV_LENGTH, -XC20P_TAG_LENGTH);
            tag = encMessage.subarray(-XC20P_TAG_LENGTH);
            _context10.next = 6;
            return xc20pDecrypter(cek)(ciphertext, tag, iv);

          case 6:
            binMessage = _context10.sent;

            if (!(binMessage === null)) {
              _context10.next = 9;
              break;
            }

            throw new Error('There was an error decoding the message!');

          case 9:
            message = bytesToString(binMessage);
            return _context10.abrupt("return", message);

          case 11:
          case "end":
            return _context10.stop();
        }
      }
    }, _callee10);
  }));

  return function decryptMessage(_x16, _x17) {
    return _ref10.apply(this, arguments);
  };
}(); // Given a private key, find the ID of the associated public key on the DID
// Note, this needs to operate on the augmented DIDDocument Version

var findVerificationMethodForKey = function findVerificationMethodForKey(didDoc, key) {
  var augmentedDIDDoc = augmentDIDDocument(didDoc);

  if (!isPublicKey(key)) {
    var keypair = makeKeypair(key);
    key = keypair.publicKey;
  } // operate on x25519 curve PubKey


  var pubkey = bytesToBase58(convertPublicKey(key.toBytes()));
  var foundVerificationMethod = (augmentedDIDDoc.verificationMethod || []).find(function (verificationMethod) {
    return verificationMethod.publicKeyBase58 === pubkey;
  });
  return foundVerificationMethod;
}; // Given a cek encrypted for fromDID, and a private key for fromDID
// Solarium uses the x25519 ECDH protocol for e2e encryption,
// which expects a key in the x25519 format (32-byte secret key)
// and type 'X25519KeyAgreementKey2019'.
// 'Sparse' DID documents on solana have an ed25519 key with the type
// 'Ed25519VerificationKey2018' by default, as this is the key type
// used to sign solana transactions. These keys are compatible with
// x25519, but require format conversion. So *unless a keyAgreement key exists*
// on the document, we artificially augment the document to include
// the converted key. This saves space on chain by avoiding the need
// to have the same key stored in two formats.

var augmentDIDDocument = function augmentDIDDocument(didDocument) {
  // key agreement key already exists, so we can use it
  if (didDocument.keyAgreement && didDocument.keyAgreement.length) return didDocument;

  if (!didDocument.verificationMethod || !didDocument.verificationMethod.length) {
    throw Error('Cannot augment DID document for x25519. The document has no keys');
  }

  var keyAgreementKeys = didDocument.verificationMethod.filter(function (key) {
    return key.type === 'Ed25519VerificationKey2018';
  }) // only apply to Ed25519
  .filter(function (key) {
    return !!key.publicKeyBase58;
  }) // we currently only support keys in base58
  .map(function (key) {
    return _extends({}, key, {
      id: key.id + '_keyAgreement',
      type: 'X25519KeyAgreementKey2019',
      publicKeyBase58: bytesToBase58(convertPublicKey(base58ToBytes(key.publicKeyBase58)))
    });
  }); // add the new key to the document

  return _extends({}, didDocument, {
    publicKey: [].concat(didDocument.verificationMethod, keyAgreementKeys),
    verificationMethod: [].concat(didDocument.verificationMethod, keyAgreementKeys),
    keyAgreement: keyAgreementKeys.map(function (key) {
      return key.id;
    })
  });
};

var AddressBook = /*#__PURE__*/function () {
  function AddressBook(data) {
    this.data = data; // TODO temporary
  } // TODO decrypt address book


  AddressBook.fromChainData =
  /*#__PURE__*/
  function () {
    var _fromChainData = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(encrypted) {
      return runtime_1.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              return _context.abrupt("return", new AddressBook(encrypted));

            case 1:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    function fromChainData(_x) {
      return _fromChainData.apply(this, arguments);
    }

    return fromChainData;
  }();

  var _proto = AddressBook.prototype;

  _proto.toChainData = function toChainData() {
    return this.data;
  };

  return AddressBook;
}();
var UserDetails = /*#__PURE__*/function () {
  function UserDetails(alias, addressBook) {
    this.alias = alias;
    this.addressBook = addressBook;
  }

  UserDetails.fromChainData = /*#__PURE__*/function () {
    var _fromChainData2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(userDetailsData) {
      var addressBook;
      return runtime_1.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              addressBook = new AddressBook(userDetailsData.addressBook);
              return _context2.abrupt("return", new UserDetails(userDetailsData.alias, addressBook));

            case 2:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));

    function fromChainData(_x2) {
      return _fromChainData2.apply(this, arguments);
    }

    return fromChainData;
  }();

  var _proto2 = UserDetails.prototype;

  _proto2.toChainData = function toChainData() {
    return new UserDetailsData({
      alias: this.alias,
      addressBook: this.addressBook.toChainData()
    });
  };

  return UserDetails;
}();

var getUserDetailsDirect = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(did, connection) {
    var didKey, encryptedUserDetails;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            didKey = didToPublicKey(did);
            _context.next = 3;
            return SolariumTransaction.getUserDetails(connection, didKey);

          case 3:
            encryptedUserDetails = _context.sent;

            if (encryptedUserDetails) {
              _context.next = 6;
              break;
            }

            return _context.abrupt("return", null);

          case 6:
            return _context.abrupt("return", UserDetails.fromChainData(encryptedUserDetails));

          case 7:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function getUserDetailsDirect(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

var userDetailsCache = /*#__PURE__*/new SolariumCache(getUserDetailsDirect);
var getUserDetails = /*#__PURE__*/userDetailsCache.load.bind(userDetailsCache);
/**
 * Create a Solarium user details account for a DID
 * @param did
 * @param owner
 * @param payer
 * @param alias
 * @param size
 * @param signCallback
 * @param cluster
 */

var createUserDetails$1 = /*#__PURE__*/function () {
  var _ref2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(did, owner, payer, alias, size, signCallback, cluster) {
    var createSignedTx, ownerDIDKey;
    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            createSignedTx = signCallback || isKeypair(payer) && isKeypair(owner) && defaultSignCallback(payer, owner);

            if (createSignedTx) {
              _context2.next = 3;
              break;
            }

            throw new Error('No payer or sign callback specified');

          case 3:
            ownerDIDKey = didToPublicKey(did);
            _context2.next = 6;
            return SolariumTransaction.createUserDetails(pubkeyOf(payer), ownerDIDKey, pubkeyOf(owner), createSignedTx, alias, size, cluster);

          case 6:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function createUserDetails(_x3, _x4, _x5, _x6, _x7, _x8, _x9) {
    return _ref2.apply(this, arguments);
  };
}();
/**
 * Update a Solarium user details account
 * @param did
 * @param connection
 * @param owner
 * @param payer
 * @param alias
 * @param addressBook
 * @param signCallback
 * @param cluster
 */

var updateUserDetails$1 = /*#__PURE__*/function () {
  var _ref3 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee3(did, connection, owner, payer, alias, addressBook, signCallback, cluster) {
    var createSignedTx, ownerDIDKey, existingUserDetails;
    return runtime_1.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            createSignedTx = signCallback || isKeypair(payer) && isKeypair(owner) && defaultSignCallback(payer, owner);

            if (createSignedTx) {
              _context3.next = 3;
              break;
            }

            throw new Error('No payer or sign callback specified');

          case 3:
            ownerDIDKey = didToPublicKey(did);
            _context3.next = 6;
            return getUserDetailsDirect(did, connection);

          case 6:
            existingUserDetails = _context3.sent;

            if (existingUserDetails) {
              _context3.next = 9;
              break;
            }

            throw new Error("DID " + did + " has no userDetails account");

          case 9:
            _context3.next = 11;
            return SolariumTransaction.updateUserDetails(ownerDIDKey, pubkeyOf(owner), createSignedTx, alias || existingUserDetails.alias, (addressBook || existingUserDetails.addressBook).toChainData(), cluster);

          case 11:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function updateUserDetails(_x10, _x11, _x12, _x13, _x14, _x15, _x16, _x17) {
    return _ref3.apply(this, arguments);
  };
}();

var Message = /*#__PURE__*/function () {
  function Message(sender, content, timestamp) {
    this.sender = sender;
    this.content = content;
    this.timestamp = timestamp;
  } // Returns the display name for the sender of this message - either an alias or did.


  var _proto = Message.prototype;

  _proto.displayableSender = function displayableSender() {
    return this.sender.alias || this.sender.did;
  };

  _proto.withContent = function withContent(content) {
    return new Message(this.sender, content, this.timestamp);
  };

  Message.build = /*#__PURE__*/function () {
    var _build = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(senderDIDKey, content, timestamp, cluster) {
      var senderDID, connection, userDetails, alias;
      return runtime_1.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              senderDID = DecentralizedIdentifier.create(senderDIDKey, currentCluster(cluster)).toString();
              connection = SolanaUtil.getConnection(cluster);
              _context.next = 4;
              return getUserDetails(senderDID, false, connection);

            case 4:
              userDetails = _context.sent;
              alias = userDetails ? userDetails.alias : undefined;
              return _context.abrupt("return", new Message({
                did: senderDID,
                alias: alias
              }, content, timestamp));

            case 7:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    function build(_x, _x2, _x3, _x4) {
      return _build.apply(this, arguments);
    }

    return build;
  }();

  return Message;
}();
var Channel = /*#__PURE__*/function () {
  function Channel(name, messages, address, cek, cluster) {
    this.name = name;
    this.messages = messages;
    this.address = address;
    this.cek = cek;
    this.cluster = cluster;
  }

  var _proto2 = Channel.prototype;

  _proto2.encrypt = /*#__PURE__*/function () {
    var _encrypt = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(message) {
      return runtime_1.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              if (this.cek) {
                _context2.next = 2;
                break;
              }

              throw new Error('Cannot encrypt, this channel was loaded without a private key, so no CEK was available');

            case 2:
              return _context2.abrupt("return", encryptMessage(message, this.cek));

            case 3:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    function encrypt(_x5) {
      return _encrypt.apply(this, arguments);
    }

    return encrypt;
  }();

  _proto2.encryptCEKForDID = /*#__PURE__*/function () {
    var _encryptCEKForDID2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee3(did) {
      return runtime_1.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              if (this.cek) {
                _context3.next = 2;
                break;
              }

              throw new Error('Cannot encrypt, this channel was loaded without a private key, so no CEK was available');

            case 2:
              return _context3.abrupt("return", encryptCEKForDID(this.cek, did));

            case 3:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3, this);
    }));

    function encryptCEKForDID$1(_x6) {
      return _encryptCEKForDID2.apply(this, arguments);
    }

    return encryptCEKForDID$1;
  }();

  _proto2.encryptCEK = /*#__PURE__*/function () {
    var _encryptCEK = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee4(verificationMethod) {
      return runtime_1.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              if (this.cek) {
                _context4.next = 2;
                break;
              }

              throw new Error('Cannot encrypt, this channel was loaded without a private key, so no CEK was available');

            case 2:
              return _context4.abrupt("return", encryptCEKForVerificationMethod(this.cek, verificationMethod));

            case 3:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4, this);
    }));

    function encryptCEK(_x7) {
      return _encryptCEK.apply(this, arguments);
    }

    return encryptCEK;
  }();

  _proto2.hasMember = /*#__PURE__*/function () {
    var _hasMember = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee5(did) {
      var cekAccount, connection, cekAccountInfo;
      return runtime_1.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              _context5.next = 2;
              return getCekAccountKey(did, this.address);

            case 2:
              cekAccount = _context5.sent;
              connection = SolanaUtil.getConnection(this.cluster);
              _context5.next = 6;
              return connection.getAccountInfo(cekAccount);

            case 6:
              cekAccountInfo = _context5.sent;
              return _context5.abrupt("return", !!cekAccountInfo);

            case 8:
            case "end":
              return _context5.stop();
          }
        }
      }, _callee5, this);
    }));

    function hasMember(_x8) {
      return _hasMember.apply(this, arguments);
    }

    return hasMember;
  }();

  Channel.fromChainData = /*#__PURE__*/function () {
    var _fromChainData = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee8(address, channelData, cekAccountData, memberDID, memberKey, cluster) {
      var getCEK, decrypt, memberDIDDocument, cek, messagePromises, messages;
      return runtime_1.wrap(function _callee8$(_context8) {
        while (1) {
          switch (_context8.prev = _context8.next) {
            case 0:
              getCEK = /*#__PURE__*/function () {
                var _ref = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee6(key) {
                  var verificationMethod;
                  return runtime_1.wrap(function _callee6$(_context6) {
                    while (1) {
                      switch (_context6.prev = _context6.next) {
                        case 0:
                          verificationMethod = findVerificationMethodForKey(memberDIDDocument, key);

                          if (verificationMethod) {
                            _context6.next = 3;
                            break;
                          }

                          throw new Error("Invalid private key for DID " + memberDIDDocument.id);

                        case 3:
                          return _context6.abrupt("return", decryptCEKs(cekAccountData.ceks, verificationMethod.id, key));

                        case 4:
                        case "end":
                          return _context6.stop();
                      }
                    }
                  }, _callee6);
                }));

                return function getCEK(_x15) {
                  return _ref.apply(this, arguments);
                };
              }();

              decrypt = /*#__PURE__*/function () {
                var _ref2 = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee7(message) {
                  var decrypted;
                  return runtime_1.wrap(function _callee7$(_context7) {
                    while (1) {
                      switch (_context7.prev = _context7.next) {
                        case 0:
                          if (cek) {
                            _context7.next = 2;
                            break;
                          }

                          return _context7.abrupt("return", message);

                        case 2:
                          _context7.next = 4;
                          return decryptMessage(message.content, cek);

                        case 4:
                          decrypted = _context7.sent;
                          return _context7.abrupt("return", message.withContent(decrypted));

                        case 6:
                        case "end":
                          return _context7.stop();
                      }
                    }
                  }, _callee7);
                }));

                return function decrypt(_x16) {
                  return _ref2.apply(this, arguments);
                };
              }();

              _context8.next = 4;
              return getDocument(memberDID);

            case 4:
              memberDIDDocument = _context8.sent;

              if (!memberKey) {
                _context8.next = 11;
                break;
              }

              _context8.next = 8;
              return getCEK(memberKey);

            case 8:
              _context8.t0 = _context8.sent;
              _context8.next = 12;
              break;

            case 11:
              _context8.t0 = undefined;

            case 12:
              cek = _context8.t0;
              messagePromises = channelData.messages.map(function (m) {
                return Message.build(m.sender.toPublicKey(), m.content, m.timestamp.toNumber(), cluster);
              }).map(function (messagePromise) {
                return messagePromise.then(decrypt);
              });
              _context8.next = 16;
              return Promise.all(messagePromises);

            case 16:
              messages = _context8.sent;
              return _context8.abrupt("return", new Channel(channelData.name, messages, address, cek, cluster));

            case 18:
            case "end":
              return _context8.stop();
          }
        }
      }, _callee8);
    }));

    function fromChainData(_x9, _x10, _x11, _x12, _x13, _x14) {
      return _fromChainData.apply(this, arguments);
    }

    return fromChainData;
  }();

  return Channel;
}();

/**
 * Gets a channel
 * @param channel
 * @param connection
 * @param memberDID
 * @param memberKey
 * @param cluster
 */

var get$1 = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(channel, connection, memberDID, memberKey, cluster) {
    var didKey, channelData, cekAccountData;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            didKey = didToPublicKey(memberDID);
            _context.next = 3;
            return SolariumTransaction.getChannelData(connection, channel);

          case 3:
            channelData = _context.sent;

            if (channelData) {
              _context.next = 6;
              break;
            }

            throw new Error("Channel not found");

          case 6:
            _context.next = 8;
            return SolariumTransaction.getCEKAccountData(connection, didKey, channel);

          case 8:
            cekAccountData = _context.sent;

            if (cekAccountData) {
              _context.next = 11;
              break;
            }

            throw new Error("No CEK account found for DID " + memberDID + ". Are they a member of the channel?");

          case 11:
            return _context.abrupt("return", Channel.fromChainData(channel, channelData, cekAccountData, memberDID, memberKey, cluster));

          case 12:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function get(_x, _x2, _x3, _x4, _x5) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Subscribe to channel updates
 * @param channel
 * @param connection
 * @param memberDID
 * @param memberKey
 * @param cluster
 */

var getStream = function getStream(channel, connection, memberDID, memberKey, cluster) {
  var didKey = didToPublicKey(memberDID);
  var cekAccountData$ = from(SolariumTransaction.getCEKAccountData(connection, didKey, channel));
  return cekAccountData$.pipe(switchMap(function (cekAccountData) {
    if (!cekAccountData) throw new Error("No CEK account found for DID " + memberDID + ". Are they a member of the channel?");
    return new Observable(function (subscriber) {
      var id = connection.onAccountChange(channel, /*#__PURE__*/function () {
        var _ref2 = _asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(accountInfo) {
          var channelData, channelObject;
          return runtime_1.wrap(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  _context2.next = 2;
                  return ChannelData.fromAccount(accountInfo.data);

                case 2:
                  channelData = _context2.sent;
                  _context2.next = 5;
                  return Channel.fromChainData(channel, channelData, cekAccountData, memberDID, memberKey, cluster);

                case 5:
                  channelObject = _context2.sent;
                  subscriber.next(channelObject);

                case 7:
                case "end":
                  return _context2.stop();
              }
            }
          }, _callee2);
        }));

        return function (_x6) {
          return _ref2.apply(this, arguments);
        };
      }()); // unsubscribe callback

      return function () {
        connection.removeAccountChangeListener(id);
      };
    });
  }));
};

/**
 * If a DID was already registered for this owner, return its document. Else create one
 * @param owner
 * @param payer
 * @param signCallback
 * @param cluster
 */

var getOrCreateDID = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(owner, payer, signCallback, cluster) {
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.prev = 0;
            debug("Looking for a DID owned by " + owner.toBase58());
            _context.next = 4;
            return get(owner, cluster);

          case 4:
            return _context.abrupt("return", _context.sent);

          case 7:
            _context.prev = 7;
            _context.t0 = _context["catch"](0);

            if (!_context.t0.message.startsWith('No DID found')) {
              _context.next = 12;
              break;
            }

            debug('No DID found - creating...');
            return _context.abrupt("return", create$1(owner, pubkeyOf(payer), undefined, signCallback, cluster));

          case 12:
            throw _context.t0;

          case 13:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[0, 7]]);
  }));

  return function getOrCreateDID(_x, _x2, _x3, _x4) {
    return _ref.apply(this, arguments);
  };
}();

var getChannel = /*#__PURE__*/function () {
  var _ref2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(owner, ownerDID, channelAddress, connection, cluster) {
    var ownerKey, channel;
    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            ownerKey = isKeypair(owner) ? owner.secretKey : undefined;
            _context2.next = 3;
            return get$1(channelAddress, connection, ownerDID, ownerKey, cluster);

          case 3:
            channel = _context2.sent;

            if (channel) {
              _context2.next = 6;
              break;
            }

            throw new Error('Error retrieving created channel');

          case 6:
            return _context2.abrupt("return", channel);

          case 7:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function getChannel(_x5, _x6, _x7, _x8, _x9) {
    return _ref2.apply(this, arguments);
  };
}();
/**
 * Creates a group channel
 * @param owner
 * @param payer
 * @param name
 * @param signCallback
 * @param cluster
 */


var createChannel = /*#__PURE__*/function () {
  var _ref3 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee3(owner, payer, name, signCallback, cluster) {
    var createSignedTx, ownerDIDDocument, didKey, ceks, connection, channelAddress;
    return runtime_1.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            createSignedTx = signCallback || isKeypair(payer) && isKeypair(owner) && defaultSignCallbackFor(payer, owner);

            if (createSignedTx) {
              _context3.next = 3;
              break;
            }

            throw new Error('No payer or sign callback specified');

          case 3:
            _context3.next = 5;
            return getOrCreateDID(pubkeyOf(owner), payer, createSignedTx, cluster);

          case 5:
            ownerDIDDocument = _context3.sent;
            didKey = didToPublicKey(ownerDIDDocument.id);
            _context3.next = 9;
            return createEncryptedCEK(ownerDIDDocument.id);

          case 9:
            ceks = _context3.sent;
            connection = SolanaUtil.getConnection(cluster);
            _context3.next = 13;
            return SolariumTransaction.createGroupChannel(connection, pubkeyOf(payer), didKey, pubkeyOf(owner), name, ceks, createSignedTx, cluster);

          case 13:
            channelAddress = _context3.sent;
            return _context3.abrupt("return", getChannel(owner, ownerDIDDocument.id, channelAddress, connection, cluster));

          case 15:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function createChannel(_x10, _x11, _x12, _x13, _x14) {
    return _ref3.apply(this, arguments);
  };
}();
/**
 * Creates a direct channel between two DIDs
 * @param owner
 * @param payer
 * @param inviteeDID
 * @param signCallback
 * @param cluster
 */

var createDirectChannel = /*#__PURE__*/function () {
  var _ref4 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee4(owner, payer, inviteeDID, signCallback, cluster) {
    var createSignedTx, ownerPubKey, ownerDIDDocument, inviteeDIDDocument, cek, ownerCEKs, inviteeCEKs, connection, channelAddress;
    return runtime_1.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            createSignedTx = signCallback || isKeypair(payer) && isKeypair(owner) && defaultSignCallback(payer, owner);

            if (createSignedTx) {
              _context4.next = 3;
              break;
            }

            throw new Error('No payer or sign callback specified');

          case 3:
            // Create the owner DID if it doesn't exist
            ownerPubKey = pubkeyOf(owner);
            _context4.next = 6;
            return getOrCreateDID(ownerPubKey, payer, createSignedTx, cluster);

          case 6:
            ownerDIDDocument = _context4.sent;
            _context4.next = 9;
            return getDocument(inviteeDID);

          case 9:
            inviteeDIDDocument = _context4.sent;
            _context4.next = 12;
            return generateCEK();

          case 12:
            cek = _context4.sent;
            _context4.next = 15;
            return encryptCEKForDID(cek, ownerDIDDocument.id);

          case 15:
            ownerCEKs = _context4.sent;
            _context4.next = 18;
            return encryptCEKForDID(cek, inviteeDIDDocument.id);

          case 18:
            inviteeCEKs = _context4.sent;
            connection = SolanaUtil.getConnection(cluster); // create the channel

            _context4.next = 22;
            return SolariumTransaction.createDirectChannel(pubkeyOf(payer), didToPublicKey(ownerDIDDocument.id), ownerPubKey, didToPublicKey(inviteeDID), ownerCEKs, inviteeCEKs, createSignedTx, cluster);

          case 22:
            channelAddress = _context4.sent;
            return _context4.abrupt("return", getChannel(owner, ownerDIDDocument.id, channelAddress, connection, cluster));

          case 24:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));

  return function createDirectChannel(_x15, _x16, _x17, _x18, _x19) {
    return _ref4.apply(this, arguments);
  };
}();

/**
 * Creates a group channel
 * @param request
 */

var create$2 = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(request) {
    var payer, owner;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            payer = toSolanaKeyMaterial(request.payer);
            owner = request.owner ? toSolanaKeyMaterial(request.owner) : pubkeyOf(payer);
            return _context.abrupt("return", createChannel(owner, payer, request.name, request.signCallback, request.cluster));

          case 3:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function create(_x) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Creates a direct chanel
 * @param request
 */

var createDirect = /*#__PURE__*/function () {
  var _ref2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(request) {
    var payer, owner;
    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            payer = toSolanaKeyMaterial(request.payer);
            owner = request.owner ? toSolanaKeyMaterial(request.owner) : pubkeyOf(payer);
            return _context2.abrupt("return", createDirectChannel(owner, payer, request.inviteeDID, request.signCallback, request.cluster));

          case 3:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function createDirect(_x2) {
    return _ref2.apply(this, arguments);
  };
}();

var postToChannel = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(connection, channelAddress, message, senderDID, signer, signCallback, payer, cluster) {
    var channel, encryptedMessage, createSignedTx, senderDIDKey;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return get$1(channelAddress, connection, senderDID, signer.secretKey, cluster);

          case 2:
            channel = _context.sent;
            _context.next = 5;
            return channel.encrypt(message);

          case 5:
            encryptedMessage = _context.sent;

            if (!(encryptedMessage.length > MESSAGE_SIZE_BYTES)) {
              _context.next = 8;
              break;
            }

            throw new Error('Message too long');

          case 8:
            createSignedTx = signCallback || payer && defaultSignCallback.apply(void 0, [payer].concat(arrayOf(signer)));

            if (createSignedTx) {
              _context.next = 11;
              break;
            }

            throw new Error('No payer or sign callback specified');

          case 11:
            senderDIDKey = didToPublicKey(senderDID);
            _context.next = 14;
            return SolariumTransaction.postMessage(channelAddress, senderDIDKey, signer.publicKey, encryptedMessage, createSignedTx, cluster);

          case 14:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function postToChannel(_x, _x2, _x3, _x4, _x5, _x6, _x7, _x8) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Post a message to a group channel
 * @param connection
 * @param senderDID
 * @param channel
 * @param signer
 * @param payer
 * @param message
 * @param signCallback
 * @param cluster
 */


var post$1 = /*#__PURE__*/function () {
  var _ref2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(connection, senderDID, channel, message, signer, payer, signCallback, cluster) {
    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return postToChannel(connection, channel, message, senderDID, signer, signCallback, payer, cluster);

          case 2:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function post(_x9, _x10, _x11, _x12, _x13, _x14, _x15, _x16) {
    return _ref2.apply(this, arguments);
  };
}();
/**
 * Post a message to a direct channel
 * @param connection
 * @param senderDID
 * @param recipientDID
 * @param signer
 * @param payer
 * @param message
 * @param signCallback
 * @param cluster
 */

var postDirect = /*#__PURE__*/function () {
  var _ref3 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee3(connection, senderDID, recipientDID, message, signer, payer, signCallback, cluster) {
    var recipientDIDKey, senderDIDKey, channelAddress;
    return runtime_1.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            recipientDIDKey = didToPublicKey(recipientDID);
            senderDIDKey = didToPublicKey(senderDID);
            _context3.next = 4;
            return getDirectChannelAccountKey(senderDIDKey, recipientDIDKey);

          case 4:
            channelAddress = _context3.sent;
            _context3.next = 7;
            return postToChannel(connection, channelAddress, message, senderDID, signer, signCallback, payer, cluster);

          case 7:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function postDirect(_x17, _x18, _x19, _x20, _x21, _x22, _x23, _x24) {
    return _ref3.apply(this, arguments);
  };
}();

/**
 * Post a message to a group channel
 * @param request
 */

var post$2 = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(request) {
    var connection, payer, payerOrUndefined, signer, senderDID;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            connection = SolanaUtil.getConnection(request.cluster);
            payer = toSolanaKeyMaterial(request.payer); // The API requires a payer either as a keypair or a pubkey but
            // In this case, the payer must either be a keypair or not present at all,
            // since the payer is only needed when request.signCallback is missing
            // this is a messy API which needs to be cleaned up TODO

            payerOrUndefined = isKeypair(payer) ? payer : undefined;
            signer = makeKeypair(request.signer);
            _context.t0 = request.senderDID;

            if (_context.t0) {
              _context.next = 9;
              break;
            }

            _context.next = 8;
            return keyToIdentifier(pubkeyOf(payer), currentCluster(request.cluster));

          case 8:
            _context.t0 = _context.sent;

          case 9:
            senderDID = _context.t0;
            return _context.abrupt("return", post$1(connection, senderDID, new PublicKey(request.channel), request.message, signer, payerOrUndefined, request.signCallback, request.cluster));

          case 11:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function post(_x) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Post a message to a direct channel
 * @param request
 */

var postDirect$1 = /*#__PURE__*/function () {
  var _ref2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(request) {
    var connection, payer, payerOrUndefined, signer, senderDID;
    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            connection = SolanaUtil.getConnection(request.cluster);
            payer = toSolanaKeyMaterial(request.payer); // The API requires a payer either as a keypair or a pubkey but
            // In this case, the payer must either be a keypair or not present at all,
            // since the payer is only needed when request.signCallback is missing
            // this is a messy API which needs to be cleaned up TODO

            payerOrUndefined = isKeypair(payer) ? payer : undefined;
            signer = makeKeypair(request.signer);
            _context2.t0 = request.senderDID;

            if (_context2.t0) {
              _context2.next = 9;
              break;
            }

            _context2.next = 8;
            return keyToIdentifier(pubkeyOf(payer), currentCluster(request.cluster));

          case 8:
            _context2.t0 = _context2.sent;

          case 9:
            senderDID = _context2.t0;
            return _context2.abrupt("return", postDirect(connection, senderDID, request.recipientDID, request.message, signer, payerOrUndefined, request.signCallback, request.cluster));

          case 11:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function postDirect(_x2) {
    return _ref2.apply(this, arguments);
  };
}();

var didFromKey = function didFromKey(request) {
  if (request.ownerDID) return Promise.resolve(request.ownerDID);
  if (request.owner) return keyToIdentifier(new PublicKey(request.owner), currentCluster(request.cluster));
  if (request.decryptionKey) return keyToIdentifier(makeKeypair(request.decryptionKey).publicKey, currentCluster(request.cluster));
  throw new Error('Unable to obtain DID from request - set either the owner or decryption key');
};
/**
 * Reads a channel
 * @param request
 */


var get$2 = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(request) {
    var connection, ownerDID;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            connection = SolanaUtil.getConnection(request.cluster);
            _context.next = 3;
            return didFromKey(request);

          case 3:
            ownerDID = _context.sent;
            return _context.abrupt("return", get$1(new PublicKey(request.channel), connection, ownerDID, request.decryptionKey, request.cluster));

          case 5:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function get(_x) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Reads a direct channel
 * @param request
 */

var getDirect = /*#__PURE__*/function () {
  var _ref2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(request) {
    var connection, ownerDID, ownerDIDKey, partnerDIDKey, channel;
    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            connection = SolanaUtil.getConnection(request.cluster);
            _context2.next = 3;
            return didFromKey(request);

          case 3:
            ownerDID = _context2.sent;
            ownerDIDKey = didToPublicKey(ownerDID);
            partnerDIDKey = didToPublicKey(request.partnerDID);
            _context2.next = 8;
            return getDirectChannelAccountKey(ownerDIDKey, partnerDIDKey);

          case 8:
            channel = _context2.sent;
            return _context2.abrupt("return", get$1(channel, connection, ownerDID, request.decryptionKey, request.cluster));

          case 10:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function getDirect(_x2) {
    return _ref2.apply(this, arguments);
  };
}();

var didFromKey$1 = function didFromKey(request) {
  if (request.memberDID) return Promise.resolve(request.memberDID);
  if (request.member) return keyToIdentifier(new PublicKey(request.member), currentCluster(request.cluster));
  return keyToIdentifier(makeKeypair(request.decryptionKey).publicKey, currentCluster(request.cluster));
};
/**
 * Reads a channel
 * @param request
 */


var read = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(request) {
    var connection, memberDID, channel;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            connection = SolanaUtil.getConnection(request.cluster);
            _context.next = 3;
            return didFromKey$1(request);

          case 3:
            memberDID = _context.sent;
            _context.next = 6;
            return get$1(new PublicKey(request.channel), connection, memberDID, request.decryptionKey, request.cluster);

          case 6:
            channel = _context.sent;

            if (channel) {
              _context.next = 9;
              break;
            }

            throw new Error('No channel found');

          case 9:
            return _context.abrupt("return", channel.messages);

          case 10:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function read(_x) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Subscribe to channel updates
 * @param request
 */

var readStream = function readStream(request) {
  var connection = SolanaUtil.getConnection(request.cluster);
  var memberDID$ = from(didFromKey$1(request));
  return memberDID$.pipe(switchMap(function (memberDID) {
    var channelInit$ = from(get$1(new PublicKey(request.channel), connection, memberDID, request.decryptionKey, request.cluster));
    var channelStream$ = getStream(new PublicKey(request.channel), connection, memberDID, request.decryptionKey, request.cluster);

    var uniqueKey = function uniqueKey(m) {
      return m.content + m.sender.did + m.timestamp;
    };

    return merge(channelInit$, channelStream$).pipe(switchMap(function (channel) {
      return channel.messages;
    })).pipe(distinct(uniqueKey)); // only emit a message once
  }));
};

var getChannel$1 = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(owner, ownerDID, channelAddress, connection, cluster) {
    var channel;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return get$1(channelAddress, connection, ownerDID, owner.secretKey, cluster);

          case 2:
            channel = _context.sent;

            if (channel) {
              _context.next = 5;
              break;
            }

            throw new Error('Error retrieving created channel');

          case 5:
            return _context.abrupt("return", channel);

          case 6:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function getChannel(_x, _x2, _x3, _x4, _x5) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Adds a DID to a channel
 * @param ownerDID
 * @param owner
 * @param payer
 * @param channel
 * @param inviteeDID
 * @param signCallback
 * @param cluster
 */


var addToChannel$1 = /*#__PURE__*/function () {
  var _ref2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(ownerDID, owner, payer, channel, inviteeDID, signCallback, cluster) {
    var createSignedTx, connection, channelForOwner, ownerDIDKey, inviteeDIDKey, inviteeIsAlreadyMember, inviteeCEKs;
    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            createSignedTx = signCallback || isKeypair(payer) && isKeypair(owner) && defaultSignCallback(payer, owner);

            if (createSignedTx) {
              _context2.next = 3;
              break;
            }

            throw new Error('No payer or sign callback specified');

          case 3:
            connection = SolanaUtil.getConnection(cluster);
            _context2.next = 6;
            return getChannel$1(owner, ownerDID, channel, connection, cluster);

          case 6:
            channelForOwner = _context2.sent;
            ownerDIDKey = didToPublicKey(ownerDID);
            inviteeDIDKey = didToPublicKey(inviteeDID);
            _context2.next = 11;
            return channelForOwner.hasMember(inviteeDIDKey);

          case 11:
            inviteeIsAlreadyMember = _context2.sent;

            if (!inviteeIsAlreadyMember) {
              _context2.next = 14;
              break;
            }

            throw new Error('Invitee DID is already a member');

          case 14:
            _context2.next = 16;
            return channelForOwner.encryptCEKForDID(inviteeDID);

          case 16:
            inviteeCEKs = _context2.sent;
            _context2.next = 19;
            return SolariumTransaction.addDIDToChannel(pubkeyOf(payer), channel, ownerDIDKey, owner.publicKey, inviteeDIDKey, inviteeCEKs, createSignedTx, cluster);

          case 19:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function addToChannel(_x6, _x7, _x8, _x9, _x10, _x11, _x12) {
    return _ref2.apply(this, arguments);
  };
}();

var didFromKey$2 = function didFromKey(request) {
  if (request.ownerDID) return Promise.resolve(request.ownerDID);
  if (request.decryptionKey) return keyToIdentifier(makeKeypair(request.decryptionKey).publicKey, currentCluster(request.cluster));
  if (request.payer) return keyToIdentifier(pubkeyOf(toSolanaKeyMaterial(request.payer)), currentCluster(request.cluster));
  throw new Error('Unable to obtain DID from request - set either the owner or decryption key');
};
/**
 * Adds a DID to a channel
 * @param request
 */


var addToChannel$2 = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(request) {
    var ownerDID;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return didFromKey$2(request);

          case 2:
            ownerDID = _context.sent;
            _context.next = 5;
            return addToChannel$1(ownerDID, makeKeypair(request.decryptionKey), toSolanaKeyMaterial(request.payer), new PublicKey(request.channel), request.inviteeDID, request.signCallback, request.cluster);

          case 5:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function addToChannel(_x) {
    return _ref.apply(this, arguments);
  };
}();

var addKey = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(did, keyIdentifier, key, signer, payer, signCallback, cluster) {
    var createSignedTx, keyDID, existingDoc, appendedCapabilityInvocation, instruction;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            // if a signCallback is not specified, both the payer and signer private keys need to be provided
            createSignedTx = signCallback || payer && isKeypair(signer) && defaultSignCallback(payer, signer);

            if (createSignedTx) {
              _context.next = 3;
              break;
            }

            throw new Error('No payer or sign callback specified');

          case 3:
            keyDID = did + '#' + keyIdentifier;
            _context.next = 6;
            return getDocument(did);

          case 6:
            existingDoc = _context.sent;
            appendedCapabilityInvocation = [].concat(existingDoc.capabilityInvocation || [], [keyDID]);
            _context.next = 10;
            return createUpdateInstruction({
              identifier: did,
              authority: pubkeyOf(signer),
              mergeBehaviour: 'Append',
              document: {
                verificationMethod: [{
                  id: keyDID,
                  type: 'Ed25519VerificationKey2018',
                  controller: did,
                  publicKeyBase58: key.toBase58()
                }],
                capabilityInvocation: appendedCapabilityInvocation
              }
            });

          case 10:
            instruction = _context.sent;
            _context.next = 13;
            return SolariumTransaction.signAndSendTransaction([instruction], createSignedTx, [], cluster);

          case 13:
            return _context.abrupt("return", getDocument(did, true));

          case 14:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function addKey(_x, _x2, _x3, _x4, _x5, _x6, _x7) {
    return _ref.apply(this, arguments);
  };
}();

var getChannel$2 = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(owner, ownerDID, channelAddress, connection, cluster) {
    var channel;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return get$1(channelAddress, connection, ownerDID, owner.secretKey, cluster);

          case 2:
            channel = _context.sent;

            if (channel) {
              _context.next = 5;
              break;
            }

            throw new Error('Error retrieving created channel');

          case 5:
            return _context.abrupt("return", channel);

          case 6:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function getChannel(_x, _x2, _x3, _x4, _x5) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Adds a key to a CEK account for a channel
 * @param ownerDID
 * @param owner
 * @param payer
 * @param channel
 * @param newKey
 * @param signCallback
 * @param cluster
 */


var updateCEKAccount = /*#__PURE__*/function () {
  var _ref2 = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee2(ownerDID, owner, payer, channel, newKey, signCallback, cluster) {
    var connection, createSignedTx, ownerDIDDocument, didKey, foundVerificationMethod, channelObject, newCEK;
    return runtime_1.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            connection = SolanaUtil.getConnection(cluster);
            createSignedTx = signCallback || isKeypair(payer) && isKeypair(owner) && defaultSignCallback(payer, owner);

            if (createSignedTx) {
              _context2.next = 4;
              break;
            }

            throw new Error('No payer or sign callback specified');

          case 4:
            _context2.next = 6;
            return getDocument(ownerDID);

          case 6:
            ownerDIDDocument = _context2.sent;
            didKey = didToPublicKey(ownerDID);
            foundVerificationMethod = findVerificationMethodForKey(ownerDIDDocument, newKey);

            if (foundVerificationMethod) {
              _context2.next = 11;
              break;
            }

            throw new Error('New key was not found on the DID');

          case 11:
            _context2.next = 13;
            return getChannel$2(owner, ownerDID, channel, connection, cluster);

          case 13:
            channelObject = _context2.sent;
            _context2.next = 16;
            return channelObject.encryptCEK(foundVerificationMethod);

          case 16:
            newCEK = _context2.sent;
            _context2.next = 19;
            return SolariumTransaction.addCEKToAccount(channel, didKey, pubkeyOf(owner), newCEK, createSignedTx, cluster);

          case 19:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function updateCEKAccount(_x6, _x7, _x8, _x9, _x10, _x11, _x12) {
    return _ref2.apply(this, arguments);
  };
}();

var didFromKey$3 = function didFromKey(request) {
  if (request.signer) return keyToIdentifier(new PublicKey(request.signer), currentCluster(request.cluster));
  return keyToIdentifier(pubkeyOf(toSolanaKeyMaterial(request.payer)), currentCluster(request.cluster));
};

var isEmpty = function isEmpty(array) {
  return !array || array.length === 0;
};
/**
 * Add an owner to an DID, and all known channels the DID is in
 * @param request
 */


var addKey$1 = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(request) {
    var did, payer, signer, payerOrUndefined, newKey, didDocument, updateChannel, channelUpdateResults;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.t0 = request.ownerDID;

            if (_context.t0) {
              _context.next = 5;
              break;
            }

            _context.next = 4;
            return didFromKey$3(request);

          case 4:
            _context.t0 = _context.sent;

          case 5:
            did = _context.t0;
            payer = toSolanaKeyMaterial(request.payer);
            signer = toSolanaKeyMaterial(request.signer); // The API requires a payer either as a keypair or a pubkey but
            // In this case, the payer must either be a keypair or not present at all,
            // since the payer is only needed when request.signCallback is missing
            // this is a messy API which needs to be cleaned up TODO

            payerOrUndefined = isKeypair(payer) ? payer : undefined;

            if (!(!isEmpty(request.channelsToUpdate) && !isKeypair(signer))) {
              _context.next = 11;
              break;
            }

            throw new Error('A decryption key is required when updating channels');

          case 11:
            newKey = new PublicKey(request.newKey);
            _context.next = 14;
            return addKey(did, request.keyIdentifier, newKey, signer, payerOrUndefined, request.signCallback, request.cluster);

          case 14:
            didDocument = _context.sent;

            updateChannel = function updateChannel(channelAddress) {
              return updateCEKAccount(did, signer, payer, new PublicKey(channelAddress), newKey, request.signCallback, request.cluster);
            };

            _context.next = 18;
            return Promise.allSettled(request.channelsToUpdate.map(updateChannel));

          case 18:
            channelUpdateResults = _context.sent;
            debug(channelUpdateResults);
            return _context.abrupt("return", didDocument);

          case 21:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function addKey(_x) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * Get a DID document for a key
 * @param request
 */

var get$3 = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(request) {
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            return _context.abrupt("return", get(new PublicKey(request.owner), request.cluster));

          case 1:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function get(_x) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * Create a DID document for a key or return an existing one
 * @param request
 */

var create$3 = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(request) {
    var payer, owner;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            payer = toSolanaKeyMaterial(request.payer);
            owner = request.owner ? toSolanaKeyMaterial(request.owner) : payer;
            _context.prev = 2;
            _context.next = 5;
            return get(pubkeyOf(owner), request.cluster);

          case 5:
            return _context.abrupt("return", _context.sent);

          case 8:
            _context.prev = 8;
            _context.t0 = _context["catch"](2);
            return _context.abrupt("return", create$1(owner, payer, request.alias, request.signCallback, request.cluster));

          case 11:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[2, 8]]);
  }));

  return function create(_x) {
    return _ref.apply(this, arguments);
  };
}();

// use common names

var didFromKey$4 = function didFromKey(request) {
  if (request.ownerDID) return Promise.resolve(request.ownerDID);
  if (request.owner) return keyToIdentifier(pubkeyOf(toSolanaKeyMaterial(request.owner)), currentCluster(request.cluster));
  if (request.payer) return keyToIdentifier(pubkeyOf(toSolanaKeyMaterial(request.payer)), currentCluster(request.cluster));
  throw new Error('Unable to obtain DID from request - set either the ownerDID, owner or payer');
};

/**
 * Create a userdetails account for a DID
 * @param request
 */

var create$4 = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(request) {
    var payer, owner, ownerDID;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            payer = toSolanaKeyMaterial(request.payer);
            owner = request.owner ? toSolanaKeyMaterial(request.owner) : pubkeyOf(payer);
            _context.next = 4;
            return didFromKey$4(request);

          case 4:
            ownerDID = _context.sent;
            _context.next = 7;
            return createUserDetails$1(ownerDID, owner, payer, request.alias, request.size, request.signCallback, request.cluster);

          case 7:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function create(_x) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * Update a userdetails account
 * @param request
 */

var update = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(request) {
    var payer, owner, ownerDID, connection;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            payer = toSolanaKeyMaterial(request.payer);
            owner = request.owner ? toSolanaKeyMaterial(request.owner) : pubkeyOf(payer);
            _context.next = 4;
            return didFromKey$4(request);

          case 4:
            ownerDID = _context.sent;
            connection = SolanaUtil.getConnection(request.cluster);
            _context.next = 8;
            return updateUserDetails$1(ownerDID, connection, owner, payer, request.alias, request.addressBook, request.signCallback, request.cluster);

          case 8:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function update(_x) {
    return _ref.apply(this, arguments);
  };
}();

/**
 * Get a DID's solarium user details
 * @param request
 */

var get$4 = /*#__PURE__*/function () {
  var _ref = /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/runtime_1.mark(function _callee(request) {
    var connection;
    return runtime_1.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            connection = SolanaUtil.getConnection(request.cluster);
            return _context.abrupt("return", getUserDetails(request.did, false, connection));

          case 2:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function get(_x) {
    return _ref.apply(this, arguments);
  };
}();

export { AddressBook, Channel, Message, STAGE, UserDetails, addKey$1 as addKey, addToChannel$2 as addToChannel, airdrop, create$2 as create, create$3 as createDID, createDirect, create$4 as createUserDetails, create as createWallet, get$2 as get, getConnection, get$3 as getDID, getDirect, get$4 as getUserDetails, post$2 as post, postDirect$1 as postDirect, read, readStream, update as updateUserDetails };
//# sourceMappingURL=solarium-js.esm.js.map
